; *******************************************************
; cette routine calibre une image NOAA au format NOAA/AVHRR
; ex: AL14082799201656202815, la georeference et
; extrait une zone definie
; Fabien Lahoche, 2001
; corrections diverses par JP Berroir
; adaptee par Milton Jonathan, 2004
; *******************************************************

pro ecoair_caliwarping_doit

  images = ['asfa', 'asdf']

  n_images = n_elements(images)

  for i=0,n_images-1 do
  begin
    image = images[i]




;
;	ouvre l'image a traiter et definit l'image de sortie
;
	name = envi_pickfile(title='select the raw NOAA image')
	if (name eq '') then return
	print, 'image selectionnee: ', name
	nameraw = name

    ;***MILTON***
    ; obtains current working directory (i.e., where the file was picked)
    lastslash = strpos(name, '\', /reverse_search)
    current_dir = strmid(name, 0, lastslash+1)


    ; obtains fid, row, line and band info
	envi_open_data_file, name, r_fid=raw_fid, /avhrr
	if (raw_fid eq -1) then return
	envi_file_query, raw_fid, ns=ns, nl=nl, nb=nb
	print, 'number of rows: ', ns
	print, 'number of lines: ', nl
	print, 'number of bands: ', nb


;
;	AVHRR CALIBRATION (bands 1 and 2)
;
	;***MILTON***
    out_name_calib = strjoin([current_dir,'calibrated'])

    ; will apply to entire image, on bands 0 and 1
	dims = [-1l, 0, ns-1, 0, nl-1]
	pos = [0,1]

	envi_check_save, /avhrr

	print, 'calibration of raw data into reflectances and radiative temperatures'

    ; performs the operation
	envi_avhrr_calibrate_doit, fid=raw_fid, pos=pos, dims=dims, $
	out_name=out_name_calib, /correct_solarz, $
	r_fid=calib_fid

	envi_check_save, /avhrr



;
;	AVHRR GEOREFERENCING (warping)
;
	;***MILTON***
    out_name_warped = strjoin([current_dir,'warped'])

    ; creates a geographic projection (obs: uses WGS84)
	;***MILTON***
	;units = envi_proj_units_translate('Degrees')
	units = envi_translate_projection_units('Degrees')
	proj = envi_proj_create(/geographic, units=units)

    ; defines pixel size for warping
	ps = [.01,.01]

	print, 'georeferencing of calibrated image'

    ; performs the operation
	envi_avhrr_warp_doit, fid=calib_fid, avhrr_fid=raw_fid, $
	dims=dims, pos=pos, out_name=out_name_warped, $
	method=6, degree=1, background=0, grid=[50,50], $
	proj=proj, pixel_size=ps, r_fid=warped_fid



;
;   IMAGE CROPPING
;
	;***MILTON***
    out_name_cropped = strjoin([current_dir,'cropped'])

	;***MILTON***
	; obtains row, line, and band info for warped image
	envi_file_query, warped_fid, ns=ns, nl=nl, nb=nb
	print, 'number de rows in warped image: ', ns
	print, 'number de lines in warped image: ', nl
	print, 'number of bands in warped image: ', nb

    ; defines cropping window in lat/long and converts it to
    ; image coordinates
	xmap=[-55.5,-52.5]
	ymap=[-17,-20]
	envi_convert_file_coordinates, warped_fid, xf, yf, xmap, ymap

    ; will apply to cropped area, without scaling, on bands 0 and 1
	dims=[-1l,xf[0],xf[1],yf[0],yf[1]]
	rfact=[1.0, 1.0]
	pos=[0,1]

	print, 'geographical coordinates to pixel coordinates'
	print, 'TLC x ',xf[0]
	print, 'BRC x ',xf[1]
	print, 'TLC y ',yf[0]
	print, 'BRC y ',yf[1]

	envi_check_save, /utility

    ; performs the operation
	resize_doit, fid=warped_fid, pos=pos, out_name=out_name_cropped, $
	dims=dims, rfact=rfact,interp=0,r_fid=cropped_fid

	;***MILTON*** (doesn't make a difference)
	;dims=dims, rfact=rfact,interp=1,r_fid=cropped_fid



;
;   IMAGE MASKING (eg: to Taquari area)
;
	;***MILTON***
	; opens mask file and generates masked output file name
   	envi_open_file, strjoin([current_dir,'masktaquari.inr']), r_fid=m_fid
	out_name_masked = strjoin([nameraw,'.out'])

	; obtains row, line, and band info for cropped image
	envi_file_query, cropped_fid, ns=nsr, nl=nlr, nb=nbr
	print, 'number of rows in cropped image: ', nsr
	print, 'number of lines in cropped image: ', nlr
	print, 'number of bands in cropped image: ', nbr

    ; will apply to entire cropped image, on every band, using the mask
    ; image's band 0 as mask
	dimsr=[-1l, 0, nsr-1, 0, nlr-1]
	posr=lindgen(nbr)
	m_pos=[0]

    ; performs the operation
	envi_mask_apply_doit, fid=cropped_fid, pos=posr, dims=dimsr, m_fid=m_fid, $
	m_pos=m_pos, value=0, out_name=out_name_masked, r_fid=masked_fid



;
;   REMOVAL OF INTERMEDIATE IMAGES
;
	print, 'removing intermediate images'
	envi_file_mng, id=calib_fid, /remove, /delete
	envi_file_mng, id=warped_fid, /remove, /delete
	envi_file_mng, id=cropped_fid, /remove, /delete


  end


end

