; *******************************************************
; cette routine calibre une image NOAA au format NOAA/AVHRR
; ex: AL14082799201656202815, la georeference et
; extrait une zone definie
; Fabien Lahoche, 2001
; corrections diverses par JP Berroir
; adaptee par Milton Jonathan, 2004
; *******************************************************

pro noaa_calib_georef

	envi, /restore_base_save_files

	; selects directory for processing
	input_dir = envi_pickfile(title='Select input directory with raw NOAA images:', /directory, /output)
	if (input_dir eq '') then return

	; selects mask file (eg. masktaquari.inr)
	;namemask = envi_pickfile(title='Select the mask file:', filter='*.inr', default='masktaquari.inr', /output, /no_change)
    ;if (namemask eq '') then return

	; finds all NOAA raw files in directory
	rawfiles = findfile('*.noaa', count=n_raw)


	output_dir = envi_pickfile(title='Select output directory for processed images:', /directory, /output, /no_change)
	if (output_dir eq '') then return


	; processes each NOAA raw file
	for i_raw=0,n_raw-1 do begin

	;
	;	ouvre l'image a traiter et definit l'image de sortie
	;
		nameraw = rawfiles[i_raw]
		print, 'image selected: ', nameraw

		; obtains fid, row, line and band info
		envi_open_data_file, nameraw, r_fid=raw_fid, /avhrr
		if (raw_fid eq -1) then return
		envi_file_query, raw_fid, ns=ns, nl=nl, nb=nb



	;
	;	AVHRR CALIBRATION (bands 1 and 2)
	;   calibrates raw data into reflectances and radiative temperatures
	;
		;***MILTON***
	    out_name_calib = strjoin([output_dir,'/Calibrated.',nameraw])

	    ; will apply to entire image, on bands 0 and 1
		dims = [-1l, 0, ns-1, 0, nl-1]
		pos = [0,1]

		envi_check_save, /avhrr


	    ; performs the operation
		envi_avhrr_calibrate_doit, fid=raw_fid, pos=pos, dims=dims, $
		out_name=out_name_calib, /correct_solarz, $
		r_fid=calib_fid

		envi_check_save, /avhrr




	;
	;	AVHRR GEOREFERENCING (warping)
	;
		;***MILTON***
	    out_name_warped = strjoin([output_dir,'/Warped.',nameraw])

	    ; creates a geographic projection (obs: uses WGS72)
		;***MILTON***
		;units = envi_proj_units_translate('Degrees')
		units = envi_translate_projection_units('Degrees')
		proj = envi_proj_create(/geographic, datum='WGS-72', units=units)

	    ; defines pixel size and grid for warping
		ps = [.01,.01]
		grid = [min([50,ns]), min([50,nl])]

	    ; performs the operation
		envi_avhrr_warp_doit, fid=calib_fid, avhrr_fid=raw_fid, $
		dims=dims, pos=pos, out_name=out_name_warped, $
		method=6, degree=1, background=0, grid=grid, $
		proj=proj, pixel_size=ps, r_fid=warped_fid


	;
	;   REMOVAL OF IMAGES
	;   removes images from disk/memory'
	;
		envi_file_mng, id=calib_fid, /remove, /delete
		;envi_file_mng, id=warped_fid, /remove, /delete
		envi_file_mng, id=warped_fid, /remove
		envi_file_mng, id=raw_fid, /remove

	endfor

end

