/*
	structure des fichiers de contours
  structure sect 0 :
    1=numero contour max, 2=nb effectif de contours, 3=nb de sect. indx
    4=nb total sect. index+data, 5=adresse 1er sect. index
    6=adresse dernier sect ind., 7=ptr de la 1ere donnee (2*256+2)
    8=ptr du prochain mot libre pour les donnees
    9= magic number (0x00ab) pour nouveau type (longueur et iext en long)
    10=surface totale d'objet (somme algebriques des 1er param. des con
    11=surf. totale des cont. "exterieurs" (somme des val. abs. des
       1ers param. des contours)
    12=nb max de parametres pour un contour donne
    33 a 64 : commentaire termine par 1/2 mot nul.

 structure sect index : 1 ptr vec sect. ind. precedent, 1 ptr vers suiv
    62 ptrs vers datas du contour (ptr num. n pour contour n) :
       valeur 256*k+l ou k=num sect et l=indice de mot dans sect

 structure secteur data : 1 ptr vers sect suivant, 63 mots de data.
   les contours sont ranges consecutivement :
     1/2 mot=lgr effective, 1/2 mot num. enveloppe
      1/2 mot nb de param, 1/2 mot indicateur de chainage (0 si datas
       consecutives, +n si n mots consecutifs pas tous occupes, -n
       si n-1 mots consecutifs suivis d'un ptr vers la suite  dans ce
        cas la suite commence par un mot indicateur de chainage)
    k mots parametres
     mots de donnees (1short x, 1 short y, valeurs >= 1))

*/

/* definitions relatives a la taille des blocs ou "secteurs" */

#define MAXCT 6
#define NBYTE 256
#define NBINT4 64
#define CT_MAGIC 0x000000ab
#define CT_MAGIC_SWAP 0xab000000

struct ct_indx {
	int4 i_prev;	/* adr. du secteur precedent */
	int4 i_next;	/* adr. secteur suivant */
	int4 i_pt[NBINT4-2];	/* ptrs vers data des contours */
};

union hdrdata {
	int4 d_l;
	struct {
		short d_lgr;	/* lgr contour en nb de mots */
		short d_env;	/* num. enveloppe */
	} d1;
	struct {
		short d_nbpar;	/* nb de parametres associes */
		short d_store;	/* indicateur de stockage des donnees :
			0 si d_lgr donnees a la suite,
			+N si d_lgr donnees sur N mots (N>= d_lgr)
			-N si le 1er mot apres param est un pointeur vers
				nouvelle zone de donnees de taille N. */
	} d2;
};

struct hdrdn {
	int4 dn_lgr;
	int4 dn_env;
	int4 dn_nbpar;
	int4 dn_store;
};

struct ct_s0 {
	int4 ct_nmax;	/* num contour max */
	int4 ct_nb;	/* nb effectif de contours */
	int4 ct_nbsi;	/* nb secteurs index */
	int4 ct_nbs;	/* nb total secteurs index+ data */
	int4 ct_si1;	/* adr 1er sect. index */
	int4 ct_sin;	/* adr dernier sect. index */
	int4 ct_dat1;	/* ptr vers 1e donnee : 2*NBYTE+2 */
	int4 ct_fdat;	/* ptr vers 1er mot de donnees libre */
	int4 ct_magic;
	int4 ct_stot;	/* surface totale */
	int4 ct_stote;	/* surf totale des cont. "exterieurs" */
	int4 ct_npmax;	/* nb max de param. par contour */
};
struct ct_b0 {
	struct ct_s0 b0_s0;
	int4 s_fill[20];
	char s_com[128];	/* commentaire termine par 0 */
};

struct fc_cont {	/* structure associee a 1 fichier de contour ouvert  */
	int4 fc_fd;	/* f.d. associe */
	int4 fc_numf;	/* numero d'ordre du fichier ouvert (0-MAXCT-1) */
	int4 n_ind;	/* numero d'ordre du bloc index courant */
	int4 s_ind;	/* adr sect index courant */
	int4 s_dat;	/* adr bloc datas courant */
	int4 *pt_datn;	/* adr dans bloc d'index du contour courant */
	int4 i_dat;	/* indice courant dans bloc data */
	int4 ct_pos;	/* indice du mot courant dans l'ensemble des datas
			du contour courant */
	int4 fc_num;	/* num contour courant */
	int4 fc_numd;	/* num contour courant pour datas */
	int4 fc_numx;	/* num contour correspondant a la fin de fichier */
	int4 ct_lgr;	/* lgr contour courant (si fc_numd == fc_num) */
	char ct_loc;	/* indic. de position : 0=indet. 1=debut, 2=zone par,
			  (ptr vers nb param ) 3= zone data */
	char fc_flag;	/* 0= cont.  vide, 1=param seuls, 2=complet */
	char fc_typ;	/* type 0= old, 1= new (que des int4) */
	char fc_swap_needed;   /* 1 si ordre des octets fichier et machine differents */
	char fc_posp;	/* indice de mot du nb param. */
	char fc_posp2;	/* indice de mot 1er param */
	char fc_res2[2];
	int4 fc_nbwr;	/* nb de blocs du fichier effectivement ecrits */
	char flg_i;	/* sect. index modifie */
	char flg_d;	/* sect data modifie */
	char flg_s0;	/* sect 0 modifie */
	char fc_access;	/* type d'access : R, W, RW, ou Modify 
			Modify = R+W+ posiibilite modifier contour */
	char fc_name[80];
	struct ct_s0 fc_s0;
	struct ct_indx fc_indx;	/* bloc index courant */
	int4 fc_dat[NBINT4];	/* bloc data */
};

#define CT_R 1
#define CT_W 2
#define CT_MOD 4

#define c_nmax fc_s0.ct_nmax
#define c_nb fc_s0.ct_nb
#define c_nbsi fc_s0.ct_nbsi
#define c_nbs fc_s0.ct_nbs
#define c_si1 fc_s0.ct_si1
#define c_sin fc_s0.ct_sin
#define c_dat1 fc_s0.ct_dat1
#define c_fdat fc_s0.ct_fdat
#define c_magic fc_s0.ct_magic
#define c_stot fc_s0.ct_stot
#define c_stote fc_s0.ct_stote
#define c_npmax fc_s0.ct_npmax

struct ct_select {
	int cnt_min, cnt_max;  /* num. min et max >= 1 */
	char *cnt_marks;  /* ptr NULL ou 1 octet/ contour (1 si actif) */
};

#define swap2(x,y) { x = y; y = ((x >> 8) & 0xff) | (x << 8); }
#define swap4(x,y) { x = y;\
y = (x << 24) | ((x & 0xff00) << 8) | ((x >> 8) & 0xff00) | ((x >> 24) & 0xff);}
