library(class)
features <- read.table('profiles.ndvi.reclassif.fit8.features.allpoints.txt', header=TRUE, fill=TRUE)

# defines number of samples from each class to be used in the analysis
nsamples = 500

# randomly selects the nsamples from each class
nclass = max(features[,'CLASS'])
samples = NULL
for (class in 1:nclass)
{
  rowsclass = (features[,'CLASS'] == class)
  featclass = features[rowsclass,]
  selected = sample(1:nrow(featclass), min(nsamples,nrow(featclass)))
  samples = rbind(samples, featclass[selected,])
}


# defines training data
values = samples[,c('mean','max','deltaMaxMin','delta150days','slopeAbsMean','nModes','begin')]

# normalizes the variables
stddev = sqrt(diag(var(values,use='pairwise.complete')))
means = mean(data.frame(values), na.rm=TRUE)
 
valuesNorm = t((t(values) - means)/stddev)
valuesNorm[is.na(valuesNorm)] = rnorm(length(valuesNorm[is.na(valuesNorm)]))


train = valuesNorm
trainLabels = samples[,'CLASS']



# defines test data
values = features[,c('mean','max','deltaMaxMin','delta150days','slopeAbsMean','nModes','begin')]

# normalizes the variables
stddev = sqrt(diag(var(values,use='pairwise.complete')))
means = mean(data.frame(values), na.rm=TRUE)
 
valuesNorm = t((t(values) - means)/stddev)
valuesNorm[is.na(valuesNorm)] = rnorm(length(valuesNorm[is.na(valuesNorm)]))

test = valuesNorm
testLabels = features[,'CLASS']



resclassif = knn(train,test,trainLabels,k=3,l=3)

n = length(resclassif)
100*sum(resclassif[1:n]==testLabels[1:n], na.rm=TRUE)/(sum(resclassif[1:n]==testLabels[1:n], na.rm=TRUE) + sum(resclassif[1:n]!=testLabels[1:n], na.rm=TRUE))

