features <- read.table('profiles.ndvi.reclassif.fit8.featu.allpoints.txt', header=TRUE, fill=TRUE)


# defines number of samples from each class to be used in the analysis
nsamples = 150

# randomly selects the nsamples from each class
nclass = max(features[,'CLASS'])
samples = NULL
for (class in 1:nclass)
{
  rowsclass = (features[,'CLASS'] == class)
  featclass = features[rowsclass,]
  selected = sample(1:nrow(featclass), min(nsamples,nrow(featclass)))
  samples = rbind(samples, featclass[selected,])
}


# separates the variable values from its classification
classif = samples[,'CLASS']
#values = samples[,4:ncol(samples)]
#values = samples[,4:17]
#values = samples[,c('mean','max','deltaMaxMin','delta150days','slopeAbsMean','nModes','datePeak','valuePeak','slopeRight','valueDeltaLeft')]
values = samples[,c('mean','max','deltaMaxMin','delta150days','slopeAbsMean','nModes','begin')]


# normalizes the variables
stddev = sqrt(diag(var(values,use='pairwise.complete')))
means = mean(data.frame(values), na.rm=TRUE)
 
valuesNorm = t((t(values) - means)/stddev)
valuesNorm[is.na(valuesNorm)] = rnorm(length(valuesNorm[is.na(valuesNorm)]))
#valuesNorm[is.na(valuesNorm)] = 0


# performs the linear discriminant analysis
res = lda(valuesNorm, classif)


# results
res

# matrix showing which variables were important for each LD function
(abs(res$scaling) > 0.5)

# sorted array showing the squared sum of the LD weights for each variable,
scores = rowsum(t(res$scaling^2), array(1,ncol(res$scaling)))
colnames(scores) <- colnames(scores)[order(scores, decreasing=TRUE)]
scores = sort(scores, decreasing=TRUE)
scores


# overview of the separation between classes provided by each LD function
# in matrix form
LDmatrix = matrix(0, nclass, nclass-1)
for (LD in 1:nclass-1)
{
  for (class in 1:nclass)
  {
    LDmatrix[class,LD] = sum(res$scaling[,LD]*res$mean[class,])
  }
}
LDmatrix

# overview of the separation between classes provided by each LD function
# in graphical form
plot.lda(res)




# ANOVA
allresults = NULL
for (feat in colnames(d))
{
  if (feat != 'CLASS')
  {
    model <- lm(d[,feat]~d$CLASS)
    allresults = rbind(allresults,anova(model))
  }
}
