#ifndef __CLASS_DEF_H__
#define __CLASS_DEF_H__

#include "linearalgebra.h"
#include "inr_io.h"

#define DEF_MAX_NUM_DISCRETE_VALUES   10


typedef struct classdef {
  // discrete feature
  double   *discreteProb;
  double   *discreteValues;
  int       nDiscreteValues;
  int       iDiscreteFeature;

  // continuous features
  matrice **contMeans;
  matrice **contCovariances;
  int      *iContFeatures;
  int       nContFeatures;

  int       nFeatures;
  int       nClass;
} classdef;


classdef *defCreate                (int nClass, int nFeatures);
classdef *defCreateFromFile        (char *filename);
void      defDestroy               (classdef **def);
int       defOutputTxt             (classdef *def, FILE *fpOutput);

double   *defGetDiscreteProb       (classdef *def, int iClass);
matrice  *defGetContMean           (classdef *def, int iClass, int val);
matrice  *defGetContCovariance     (classdef *def, int iClass, int val);
int       defSetContCovariance     (classdef *def, int iClass, int val, matrice *cov);
int       defGetDiscreteValueIndex (classdef *def, double value);

int       defGetPoint              (classdef *def, int pos, unsigned char *dataInput,
                                    int size_x, int size_y, InrType typeInput,
                                    double *contValues, int *iDiscrete);

int       defProcessDiscrete(classdef *def, unsigned char *classif, unsigned char *dataInput,
                             int size_x, int size_y, int size_z,  InrType typeInput);

int       defComputeDiscreteProb(classdef *def, unsigned char *classif, unsigned char *dataInput,
                                 int size_x, int size_y, int size_z,  InrType typeInput);

// auxiliary 
int       defDefineDiscreteFeature(classdef *def, int iFeature, double *values, int nValues);

int       defCheckDiscreteFeature(int iFeature, unsigned char *ROI, unsigned char *dataInput,
                                  int size_x, int size_y, int size_z, InrType typeInput,
                                  double *discreteValues, int *nDiscreteValues);


#endif

