/*
   svd_util.c

   estimation des reflectances NOAA des classes
   methode lineaire:
   un pixel NOAA est la somme surface_classe*reflectance_classe
   pour les classes contenues
*/
#include <stdio.h>
#include <math.h>
#include <inrimage/image.h>

#include <stdlib.h>

#include "svd.h"

#define NR_END 1
#define FREE_ARG char*
#define TOL 0.0001
#define ntirmax 100
static float sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
static float maxarg1, maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1) > (maxarg2) ?\
        (maxarg1) : (maxarg2))
static int iminarg1, iminarg2;
#define IMIN(a,b) (iminarg1=(a),iminarg2=(b),(iminarg1) < (iminarg2) ?\
        (iminarg1) : (iminarg2))


float *
vector (long nl, long nh)
/* allocation d'un vecteur de reels v[nl..nh] */
{
  float *v;

  v = (float *) malloc ((size_t) ((nh - nl + 1 + NR_END) * sizeof (float)));
  if (!v)
    {
      printf ("Memory allocation failed\n");
      exit (1);
    }
  return v - nl + NR_END;
}

float **
matrix (long nrl, long nrh, long ncl, long nch)
/* allocation d'une matrice de reels m=[nrl..nrh][ncl..nch] */
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  float **m;

  /* allocation des pointeurs sur les lignes */
  m = (float **) malloc ((size_t) ((nrow + NR_END) * sizeof (float *)));
  if (!m)
    {
      printf ("Memory allocation failed\n");
      exit (1);
    }
  m += NR_END;
  m -= nrl;

  /* allocation des lignes et leurs  pointeurs */
  m[nrl] =
    (float *) malloc ((size_t) ((nrow * ncol + NR_END) * sizeof (float)));
  if (!m[nrl])
    {
      printf ("Memory allocation failed\n");
      exit (1);
    }
  m[nrl] += NR_END;
  m[nrl] -= ncl;

  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;

  /* return pointer to array of pointers to rows */
  return m;
}

void
free_vector (v, nl, nh)
     float *v;
     long nh, nl;
/* liberer le vecteur de float cree par vector() */
{
  free ((FREE_ARG) (v + nl - NR_END));
}

void
free_matrix (m, nrl, nrh, ncl, nch)
     float **m;
     long nch, ncl, nrh, nrl;
/* libere la matrice de float cree par matrix() */
{
  free ((FREE_ARG) (m[nrl] + ncl - NR_END));
  free ((FREE_ARG) (m + nrl - NR_END));
}

float
pythag (float a, float b)
{                                /* calcule la racine carre de (a2 + b2) */

  float absa, absb;
  absa = (float) fabs ((double) a);
  absb = (float) fabs ((double) b);
  if (absa > absb)
    return absa * (float) sqrt ((double) (1.0 + SQR (absb / absa)));
  else
    return (absb ==
            0.0 ? 0.0 : absb *
            (float) sqrt ((double) (1.0 + SQR (absa / absb))));
}

/*****************************************************************************/
void
mul_2matrice (M1, lig1, col1, M2, lig2, col2, Mres)
     float **M1, **M2, **Mres;
     int lig1, col1, lig2, col2;
{
  int i, k, l;
  if (col1 != lig2)
    {
      printf ("cannot multiply matrices\n");
      exit (-1);
    }
  for (i = 1; i <= lig1; i++)
    for (k = 1; k <= col2; k++)
      {
        Mres[i][k] = 0.0;
        for (l = 1; l <= col1; l++)
          Mres[i][k] += M1[i][l] * M2[l][k];
      }
}

/*****************************************************************************/
void
svdcmp (float **a, int m, int n, float w[], float **v)
{
  /*
     etant donnee une matrice a[1..m][1..n] cette routine
     calcule sa decomposition en valeurs singulieres
   */

  float pythag (float a, float b);
  int flag, i, its, j, jj, k, l=0, nm=0;
  float anorm, c, f, g, h, s, scale, x, y, z, *rv1;

  rv1 = vector (1, n);
  g = scale = anorm = 0.0;
  for (i = 1; i <= n; i++)
    {
      l = i + 1;
      rv1[i] = scale * g;
      g = s = scale = 0.0;
      if (i <= m)
        {
          for (k = i; k <= m; k++)
            scale += (float) fabs ((double) a[k][i]);
          if (scale)
            {
              for (k = i; k <= m; k++)
                {
                  a[k][i] /= scale;
                  s += a[k][i] * a[k][i];
                }
              f = a[i][i];
              g = -SIGN ((float) sqrt ((double) s), f);
              h = f * g - s;
              a[i][i] = f - g;
              for (j = l; j <= n; j++)
                {
                  for (s = 0.0, k = i; k <= m; k++)
                    s += a[k][i] * a[k][j];
                  f = s / h;
                  for (k = i; k <= m; k++)
                    a[k][j] += f * a[k][i];
                }
              for (k = i; k <= m; k++)
                a[k][i] *= scale;
            }
        }
      w[i] = scale * g;
      g = s = scale = 0.0;
      if (i <= m && i != n)
        {
          for (k = l; k <= n; k++)
            scale += (float) fabs ((double) a[i][k]);
          if (scale)
            {
              for (k = l; k <= n; k++)
                {
                  a[i][k] /= scale;
                  s += a[i][k] * a[i][k];
                }
              f = a[i][l];
              g = -SIGN ((float) sqrt ((double) s), f);
              h = f * g - s;
              a[i][l] = f - g;
              for (k = l; k <= n; k++)
                rv1[k] = a[i][k] / h;
              for (j = l; j <= m; j++)
                {
                  for (s = 0.0, k = l; k <= n; k++)
                    s += a[j][k] * a[i][k];
                  for (k = l; k <= n; k++)
                    a[j][k] += s * rv1[k];
                }
              for (k = l; k <= n; k++)
                a[i][k] *= scale;
            }
        }
      anorm =
        FMAX (anorm,
              ((float) fabs ((double) w[i]) +
               (float) fabs ((double) rv1[i])));
    }
  for (i = n; i >= 1; i--)
    {
      if (i < n)
        {
          if (g)
            {
              for (j = l; j <= n; j++)
                v[j][i] = (a[i][j] / a[i][l]) / g;
              for (j = l; j <= n; j++)
                {
                  for (s = 0.0, k = l; k <= n; k++)
                    s += a[i][k] * v[k][j];
                  for (k = l; k <= n; k++)
                    v[k][j] += s * v[k][i];
                }
            }
          for (j = l; j <= n; j++)
            v[i][j] = v[j][i] = 0.0;
        }
      v[i][i] = 1.0;
      g = rv1[i];
      l = i;
    }
  for (i = IMIN (m, n); i >= 1; i--)
    {
      l = i + 1;
      g = w[i];
      for (j = l; j <= n; j++)
        a[i][j] = 0.0;
      if (g)
        {
          g = 1.0 / g;
          for (j = l; j <= n; j++)
            {
              for (s = 0.0, k = l; k <= m; k++)
                s += a[k][i] * a[k][j];
              f = (s / a[i][i]) * g;
              for (k = i; k <= m; k++)
                a[k][j] += f * a[k][i];
            }
          for (j = i; j <= m; j++)
            a[j][i] *= g;
        }
      else
        for (j = i; j <= m; j++)
          a[j][i] = 0.0;
      ++a[i][i];
    }
  for (k = n; k >= 1; k--)
    {
      for (its = 1; its <= 30; its++)
        {
          flag = 1;
          for (l = k; l >= 1; l--)
            {
              nm = l - 1;
              if ((float) (fabs ((double) rv1[l]) + anorm) == anorm)
                {
                  flag = 0;
                  break;
                }
              if ((float) (fabs ((double) w[nm]) + anorm) == anorm)
                break;
            }
          if (flag)
            {
              c = 0.0;
              s = 1.0;
              for (i = l; i <= k; i++)
                {
                  f = s * rv1[i];
                  rv1[i] = c * rv1[i];
                  if ((float) (fabs ((double) f) + anorm) == anorm)
                    break;
                  g = w[i];
                  h = pythag (f, g);
                  w[i] = h;
                  h = 1.0 / h;
                  c = g * h;
                  s = -f * h;
                  for (j = 1; j <= m; j++)
                    {
                      y = a[j][nm];
                      z = a[j][i];
                      a[j][nm] = y * c + z * s;
                      a[j][i] = z * c - y * s;
                    }
                }
            }
          z = w[k];
          if (l == k)
            {
              if (z < 0.0)
                {
                  w[k] = -z;
                  for (j = 1; j <= n; j++)
                    v[j][k] = -v[j][k];
                }
              break;
            }
          if (its == 30)
            {
              printf ("no convergence in 30 svdcmp iterations");
              exit (1);
            }
          x = w[l];
          nm = k - 1;
          y = w[nm];
          g = rv1[nm];
          h = rv1[k];
          f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0 * h * y);
          g = pythag (f, 1.0);
          f = ((x - z) * (x + z) + h * ((y / (f + SIGN (g, f))) - h)) / x;
          c = s = 1.0;
          for (j = l; j <= nm; j++)
            {
              i = j + 1;
              g = rv1[i];
              y = w[i];
              h = s * g;
              g = c * g;
              z = pythag (f, h);
              rv1[j] = z;
              c = f / z;
              s = h / z;
              f = x * c + g * s;
              g = g * c - x * s;
              h = y * s;
              y *= c;
              for (jj = 1; jj <= n; jj++)
                {
                  x = v[jj][j];
                  z = v[jj][i];
                  v[jj][j] = x * c + z * s;
                  v[jj][i] = z * c - x * s;
                }
              z = pythag (f, h);
              w[j] = z;
              if (z)
                {
                  z = 1.0 / z;
                  c = f * z;
                  s = h * z;
                }
              f = c * g + s * y;
              x = c * y - s * g;
              for (jj = 1; jj <= m; jj++)
                {
                  y = a[jj][j];
                  z = a[jj][i];
                  a[jj][j] = y * c + z * s;
                  a[jj][i] = z * c - y * s;
                }
            }
          rv1[l] = 0.0;
          rv1[k] = f;
          w[k] = x;
        }
    }

  free_vector (rv1, 1, n);
}


int
my_rand (int n)
{
  //*** MILTON ***
  #if (defined(WIN32) || defined(_WIN32))
  return (int) ((double) n * rand()/RAND_MAX);
  #else
  return (int) (drand48 () * (double) n);
  #endif
}

int
compter_pixels (unsigned char *mask, int taille)
{
  int nb = 0, i;

  for (i = 0; i < taille; i++)
    if (mask[i])
      nb++;
  return (nb);
}

void
svbksb (float **u, float w[], float **v, int m, int n, float b[], float x[])
{
  /*
     resoud A.X=B , A etant reprsent par les vecteurs
     u[1..m][..n],w[1..n],v[1..n][1..n] resultats de la routine svdcmp.
     Le resultat de la resolution du systeme est stocke dans x[1..m]
   */
  int jj, j, i;
  float s, *tmp;

  tmp = vector (1, n);
  for (j = 1; j <= n; j++)
    {
      s = 0.0;
      if (w[j])
        {
          for (i = 1; i <= m; i++)
            s += u[i][j] * b[i];
          s /= w[j];
        }
      tmp[j] = s;
    }
  for (j = 1; j <= n; j++)
    {
      s = 0.0;
      for (jj = 1; jj <= n; jj++)
        s += v[j][jj] * tmp[jj];
      x[j] = s;
    }
  free_vector (tmp, 1, n);
}

/* Conditionnement de la matrice des proportions
 */

void
conditionnement (float **mprop, int nbpix, int nbc, float **V, float *W)
{
  int k, comptpar, ind_p=0, i;
  float wmax, wmin, *vi, cond, mm;


  vi = vector (1, nbc);

  svdcmp (mprop, nbpix, nbc, W, V);
  wmax = 0.0;
  wmin = 10000;
  for (k = 1; k <= nbc; k++)
    {
      if (W[k] > wmax)
        wmax = W[k];
      if ((W[k] < wmin) && (wmin > 0))
        wmin = W[k];
    }
  if (wmin == 0.0)
    {
      printf
        ("Warning! One singular value is zero. Most likely there is a class with no representant\n");
            //*** MILTON ***
      exit (-1);
      //exit ();
    }


  cond = wmax / wmin;

  for (k = 1; k <= nbc; k++)
    vi[k] = wmax / W[k];
  if (cond < s1)
    printf ("All parameters identified\n");
  else
    for (k = 1; k <= nbc; k++)
      {
        if (vi[k] > s1)
          {
            if (vi[k] > s2)
              {
                printf ("Singular value %f too small\n", W[k]);
                mm = 0.0;
                for (i = 1; i <= nbc; i++)
                  if (fabsf (V[i][k]) >= mm)
                    {
                      mm = fabsf (V[i][k]);
                      ind_p = i;
                    }
                comptpar = 0;
                for (i = 1; i <= nbc; i++)
                  if (fabsf (mm / V[i][k]) > s3)
                    comptpar++;
                if (comptpar >= 2 * nbc / 3)
                  {
                    printf ("parameter %d not identified\n", ind_p);
                  }
                else
                  for (i = 1; i <= nbc; i++)
                    if (fabsf (mm / V[i][k]) > s3)
                      {
                        printf ("parameter %d not identified\n", i);
                      }
              }
            else
              {
                mm = 0;
                for (i = 1; i <= nbc; i++)
                  if (fabsf (V[i][k]) > mm)
                    {
                      mm = fabsf (V[i][k]);
                      ind_p = i;
                    }
                comptpar = 0;
                for (i = 1; i <= nbc; i++)
                  if (fabsf (mm / V[i][k]) > s3)
                    comptpar++;
                if (comptpar >= 2 * nbc / 3)
                  printf ("parameter %d not reliable\n", ind_p);
                /*    else
                   for (i=1;i<=nbc;i++)
                   if (fabsf(mm/V[i][k])> s3)
                   printf("parametre %d est peu fiable\n",i); */

              }
          }
      }
  free_vector (vi, 1, nbc);
}
