/*
  conversion des dates yyyymm et yymmdd en jours
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <locale.h>
#include "dlr_dates.h"

#define _XOPEN_SOURCE

static char dfmt[10];

static struct tm * ttm ;
static time_t tt ;



void init_date()
{
  //  (void)setlocale(LC_ALL, "");
  time(&tt);
  ttm = localtime(&tt);
}


int setdatemode( int date ) 
{
  char sdate[10] ;

  sprintf(sdate,"%d",date);

  switch (strlen(sdate) ){
  case 6: //yyyymm
    strcpy (dfmt,"%Y%m"); 
    break;
  case 7: //yyyyddd
    strcpy (dfmt,"%Y%j");
    break;
  case 8: //yyyymmdd
    strcpy(dfmt,"%Y%m%d");
    break;
  default:
    fprintf(stderr,"impossible de determiner le format des dates\n");
    exit(1);
  }
  return 1;
}


// convert a YYYYMM or YYYYMMDD date into year and julian day
void convertdate( int date, int *y, int *j )
{
  char sdate[10];

  sprintf(sdate,"%d",date);

  if (!strptime(sdate,dfmt,ttm)){
    fprintf(stderr,"date %s invalide\n",sdate);
    exit(1);
  }
  *y = ttm->tm_year ;
  *j = ttm->tm_yday ;
}

int daydiff( int y1, int d1, int y2, int d2 )
{
  int y ;

  if (y1==y2) return d2-d1 ;
  for (y=y1;y<y2;y++){
    if (isleap(y)) d2+=366;
    else d2+=365;
  }
  return d2-d1;
}

void  setoutdates(int *dates,int*outdates,int idmin,int idmax)
{
  int t ;
  int y0,d0,y1,d1;

  init_date() ;
  setdatemode( dates[0] );

  convertdate(dates[idmin] , &y0, & d0 ) ;

  outdates[idmin]=d0 ;

  for (t=idmin+1;t<=idmax;t++) {
    convertdate(dates[t],&y1,&d1);
    outdates[t]=d0+daydiff(y0,d0,y1,d1);
  }
}

/*
 *  defineDates
 *
 *  Builds date vector from a datefile if it is specified
 *  If not, date vector will be [0, 1, .., nDates]
 */
int *defineDates(int hasFile, char *fnameDates, int nDates)
{
  int *dates = (int *)malloc(nDates*sizeof(int));
  int *outdates = (int *)malloc(nDates*sizeof(int));
  int  date;
  int  iDate;
  FILE *fpDates;
  
  if (!hasFile)
  {
    // case 1: no dates file specified
    for (iDate=0; iDate<nDates; iDate++)
      dates[iDate] = iDate;
    return dates;
  }
  else
  {
    // case 2: dates file specified
    fpDates = fopen(fnameDates, "rt");
    if (!fpDates)
    {
      fprintf(stderr, "Error: could not open dates file '%s'.\n", fnameDates);
      free(dates);
      return NULL;
    }
    for (iDate=0; iDate<nDates; iDate++)
    {
      if (!fscanf(fpDates, "%d", &date))
      {
        fprintf(stderr, "Error: not enough dates (%d) in dates file '%s'.\n", iDate, fnameDates);
        free(dates);
        return NULL;
      }
      dates[iDate] = date;
    }
  }
  setoutdates(dates,outdates,0,nDates-1);
/*    for (iDate=0; iDate<nDates; iDate++) */
/*      fprintf(stderr,"dates %d=%d\n",iDate,outdates[iDate]); */
  free(dates);
  return outdates;
}

