#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "file_tools.h"



/*
 *  ftGetDirDivider
 * 
 *  Returns '\\' for Windows systems, ':' for Mac systems, '/' otherwise
 */
char ftGetDirDivider(void)
{
  #if defined(WIN32) || defined(_WIN32)
  return '\\';
  #elif defined(MACOS)
  return ':';
  #else
  return '/';
  #endif
}


/*
 *  ftFixDirName
 * 
 *  Appends a final directory divider (e.g., a slash '/' or backslash '\\') to
 *  the directory name, if missing
 *  (eg. directory "/usr" becomes "/usr/")
 *  Important: 'dirname' buffer should be large enough to accomodate the final
 *  divider
 */
char *ftFixDirName(char *dirname)
{
  size_t len = strlen(dirname);
  if (dirname[len-1] != ftGetDirDivider())
  {
    dirname[len]   = ftGetDirDivider();
    dirname[len+1] = '\0';
  }
  
  return dirname;
}


/*
 *  ftDecodeFileName
 * 
 *  Analyzes a full-path file name and extracts the directory and file names
 *  (eg. from "/usr/bin/test/input.img" to "/usr/bin/test/" and "input.img")
 *  'dirname' and 'filename' buffers should be large enough to accomodate the
 *  data
 */
char *ftDecodeFileName(char *fullname, char *dirname, char *filename)
{
  char *ptrDir;
  char *buf = (char *)malloc(strlen(fullname)*sizeof(char));
  
  strcpy(buf, fullname);

  //if (fullname != dirname)
  //  strcpy(dirname, fullname);

  // extracts information
  ptrDir = strrchr(buf, ftGetDirDivider());
  if (ptrDir)
  {
    if (filename)
      strcpy(filename, ptrDir+1);
    ptrDir[1] = '\0';
    if (dirname)
      strcpy(dirname, buf);
  }
  else
  {
    //if (filename != fullname)
    //  strcpy(filename, fullname);
    if (filename)
      strcpy(filename, buf);
    if (dirname)
      sprintf(dirname, ".%c", ftGetDirDivider());
  }
  free(buf);
  return filename;
}


/*
 *  ftChangeExtension
 * 
 *  Changes the extension of a file name (eg. from "input.inr" to input.hdr")
 *  The 'outputname' buffer should be large enough to accomodate the
 *  new name.
 *  The 'ext' string should include the initial '.' character, if desired.
 */
char *ftChangeExtension(char *inputname, char *outputname, char *ext)
{
  char *ptrExt;

  strcpy(outputname, inputname);
  ptrExt = strrchr(outputname, '.');
  if (ptrExt)
    ptrExt[0] = '\0';
    
  return strcat(outputname, ext);
}


/*
 *  ftFileExists
 * 
 *  Given a filename, returns whether that file actually exists.
 */
int ftFileExists(char *filename)
{
  FILE *fp = fopen(filename, "rb");
  if (!fp)
    return 0;
  else
  {
    fclose(fp);
    return 1;
  }
}


/*
 *  ftCountLineValues
 * 
 *  Counts the number of values (numbers) in a text file until the next new
 *  line character
 */
int ftCountLineValues(FILE *fp)
{
  //*** NOT IMPLEMENTED ***
  //int   ret;
  //char *buf = (char *)malloc(10 * sizeof(char));
  return -1;
}

