#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <inrimage/image.h>
#include "inr_io.h"


#define FNAMESIZE  1024
#define MIN(x,y) ((x<y)?x:y)
#define MAX(x,y) ((x>y)?x:y)



static char Ucmd[] = "-ifile inr_file -ofile txt_file\n";

static char Udetail[] =
"Converts an INRIMAGE 2D image into a text file containing a matrix of numbers.\n\
\nInput:\n\
\t-ifile\tFile name of the input INRIMAGE file\n\
\nOutput:\n\
\t-ofile\tFile name of the output text file\n";





int main (int argc, char **argv)
{
  char fnameInput[FNAMESIZE];
  char fnameOutput[FNAMESIZE];
  InrType type;
  int     typesize;

  FILE *fpOutput;

  unsigned char *dataInput;

  int size_x, size_y, size_z;
  int x, y;
  double value;
  

  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile", "%s", fnameOutput))
    iusage_ (Ucmd, Udetail);
  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);


  // reads header information for all input data and checks if they are appropriate
  if (!inrReadHeader(fnameInput, &size_x, &size_y, &size_z, &type))
  {
    fprintf(stderr, "Error: could not read header information for input file '%s'.\n", fnameInput);
    return 0;
  }
  if (size_z != 1)
  {
    fprintf(stderr, "Error: Input image '%s' has to be a single-layer image.\n", fnameInput);
    return 0;
  }
 
  // opens output data file
  fpOutput = fopen(fnameOutput, "wt");
  if (!fpOutput)
  {
    fprintf(stderr, "Error: could not read output file '%s'.\n", fnameOutput);
    return 0;
  }

  // reads input data
  dataInput = inrReadDataSequence(fnameInput, &size_x, &size_y, &size_z, &type);
  if (!dataInput)
  {
    fprintf (stderr, "Error: could not read input file '%s'.\n", fnameInput);
    return 0;
  }
  typesize = InrTypeSizes[type]/8;
  

  // processes the data
  for (y=0; y<size_y; y++)
  {
    for (x=0; x<size_x; x++)
    {
      value = inrGetDataValue(dataInput, type, x + y*size_x);
      fprintf(fpOutput, "%.10lf\t", value);
    }
    fprintf(fpOutput, "\n");
  }
  fclose(fpOutput);

  
  free(dataInput);
  fprintf(stderr, "Image successfully converted.\n");

  return 1;
}
