#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inrimage/image.h>
#include "file_tools.h"
#include "inr_io.h"

#define FNAMESIZE    1024


static char Ucmd[] =
"-ifile image -ofile image_converted -nbytes value [-float] [-signed] [-unsigned]";

static char Udetail[] =
"\nConverts an INRIMAGE image to another type\n";




int main(int argc, char *argv[])
{
  int      okFloat, okSigned, okUnsigned;
  char     fnameIn[FNAMESIZE];
  char     fnameOut[FNAMESIZE];

  unsigned char *data_in;
  unsigned char *data_out;
  InrType typeInput;
  InrType typeOutput;
  int  typesizeInput;
  int  typesizeOutput;
  char typenameOutput[FNAMESIZE];
  int  size_x, size_y, size_z;
  int  z;
  int  i;
  double val;
  int nBytes;
  
  InrGeorefInfo georef;
  

  // initializes inrimage
  inr_init (argc, argv, "1.0", Ucmd, Udetail);

  // checks arguments
  if (!igetopt1 ("-ifile", "%s", fnameIn))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile" , "%s", fnameOut))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-nbytes", "%d", &nBytes))
    iusage_ (Ucmd, Udetail);
  okFloat    = igetopt0 ("-float");
  okSigned   = igetopt0 ("-signed");
  okUnsigned = igetopt0 ("-unsigned");
  if (tstopts ())
    iusage_ (Ucmd, Udetail);
    
  if (okFloat && (okSigned || okUnsigned))
    iusage_ (Ucmd, Udetail);
  if (okSigned && okUnsigned)
    iusage_ (Ucmd, Udetail);

  // defines output file type
  if (okFloat)
    sprintf(typenameOutput, "float");
  else if (okSigned)
    sprintf(typenameOutput, "signed fixed");
  else
    sprintf(typenameOutput, "unsigned fixed");
  if (!inrGetType(typenameOutput, nBytes*8, &typeOutput))
  {
    fprintf(stderr, "Error: specified output data type is not supported.\n");
    return 0;
  }

  if (!inrReadHeader(fnameIn, &size_x, &size_y, &size_z, &typeInput))
  {
    fprintf(stderr, "Error: could not read header for image '%s'.\n", fnameIn);
    return 0;
  }

  typesizeInput  = InrTypeSizes[typeInput]/8;
  typesizeOutput = InrTypeSizes[typeOutput]/8;
  data_in  = (unsigned char *)malloc(size_x*size_y*typesizeInput*sizeof(unsigned char));    
  data_out = (unsigned char *)malloc(size_x*size_y*typesizeOutput*sizeof(unsigned char));
  if (!data_in || !data_out)
  {
    fprintf(stderr, "Error: could not allocate memory.\n");
    return 0;
  }
  
  if (!inrWriteHeader(fnameOut, size_x, size_y, size_z, typeOutput))
  {
    fprintf(stderr, "Error: could write to output file '%s'.\n", fnameOut);
    return 0;
  }
  
  for (z=0; z<size_z; z++)
  {
    fprintf(stderr, "Processing layer %d of %d...\r", z+1, size_z);
    if (!inrReadFrame(fnameIn, z, data_in))
    {
      fprintf(stderr, "Error: could not read frame %d for image '%s'.\n", z, fnameIn);
      free(data_in);
      free(data_out);
      return 0;
    }
    
    for (i=0; i<size_x*size_y; i++)
    {
      val = inrGetDataValue(data_in, typeInput, i);
      inrPutDataValue(data_out, typeOutput, i, val);
    }
    
    if (!inrWriteFrame(fnameOut, z, (unsigned char *)data_out))
    {
      fprintf(stderr, "Error: could not write frame %d for output image.\n", z);
      free(data_in);
      free(data_out);
      return 0;
    }
  }
  
  if (inrReadGeoref(fnameIn, &georef))
    inrWriteGeoref(fnameOut, georef);
  else
    inrWriteEnviHdr(fnameOut);
    
  free(data_in);
  free(data_out);
  printf("INRIMAGE image successfully converted.\n");

  return 1;
}

