#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <inrimage/image.h>
#include "inr_io.h"
#include "file_tools.h"


#define FNAMESIZE  1024
#define MIN(x,y) ((x<y)?x:y)
#define MAX(x,y) ((x>y)?x:y)



static char Ucmd[] = "-ifile inr_file -ofile layer_file [-layer index] [-all]";

static char Udetail[] =
"Extracts a layer from an INRIMAGE file.\n\
\nInput:\n\
\t-ifile\tFile name of the input INRIMAGE file\n\
\nOutput:\n\
\t-ofile\tFile name of the output INRIMAGE file containing the desired\n\
\t\tlayer\n\
\nOptional:\n\
\t-layer\tIndex of one layer to be extracted (0..size_z-1)\n\
\t-all\tIndicates that all layers should be extracted\n";






int main (int argc, char **argv)
{
  char fnameInput[FNAMESIZE];
  char fnameOutput[FNAMESIZE];
  char fnameLayer[FNAMESIZE];
  int  okLayer, okAll;
  InrType type;
  int     typesize;
  InrGeorefInfo georef;

  int size_x, size_y, size_z;
  int nLayers;
  int iLayer;
  int chosenLayer;
  int layer;

  unsigned char *data;

  

  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile", "%s", fnameOutput))
    iusage_ (Ucmd, Udetail);
  okLayer = igetopt1 ("-layer", "%d", &chosenLayer);
  okAll   = igetopt0 ("-all");
  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);

  if (!okLayer && !okAll)
    iusage_ (Ucmd, Udetail);
  
  // reads header information for all input data and checks if they are appropriate
  if (!inrReadHeader(fnameInput, &size_x, &size_y, &size_z, &type))
  {
    fprintf(stderr, "Error: could not read header information for input file '%s'.\n", fnameInput);
    return 0;
  }
  if (!okAll && (chosenLayer < 0 || chosenLayer >= size_z))
  {
    fprintf(stderr, "Error: Specified layer '%d' not present in input image '%s'.\n", chosenLayer, fnameInput);
    fprintf(stderr, "       Image contains layers 0..%d.\n", size_z-1);
    return 0;
  }

  if (okAll)
    nLayers = size_z;
  else
    nLayers = 1;


  // allocates memory for the input and output data
  typesize = InrTypeSizes[type]/8;
  data     = (unsigned char *) malloc (size_x*size_y * typesize * sizeof(unsigned char));
  

  for (iLayer=0; iLayer<nLayers; iLayer++)
  {
    if (!okAll)
    {
      layer = chosenLayer;
      strcpy(fnameLayer, fnameOutput);
    }
    else
    {
      char ext[FNAMESIZE];
      layer = iLayer;
      sprintf(ext, ".%03d.inr", iLayer);
      ftChangeExtension(fnameOutput, fnameLayer, ext);
    }
      
    // writes output file header
    if (!inrWriteHeader(fnameLayer, size_x, size_y, 1, type))
    {
      printf("Error: could not write header information for output file '%s'.\n", fnameLayer);
      return 0;
    }
  
    // reads specified layer
    if (!inrReadFrame(fnameInput, layer, data))
    {
      fprintf(stderr, "Error: could not read frame %d for input image '%s'.\n", layer, fnameInput);
      return 0;
    }
  
    // writes output data layer
    if (!inrWriteFrame(fnameLayer, 0, (unsigned char*)data))
    {
      fprintf(stderr, "Error: could not write frame %d for output image '%s'.\n", 0, fnameLayer);
      return 0;
    }

    // transfers georeference info if available
    if (inrReadGeoref(fnameInput, &georef))
      inrWriteGeoref(fnameLayer, georef);
    else
      inrWriteEnviHdr(fnameLayer);
  }

  free(data);
  fprintf(stderr, "%d layer(s) successfully extracted.\n", nLayers);

  return 1;
}
