#ifndef __INR_IO_H__
#define __INR_IO_H__


#define INRHDRBUFSIZE 4096

// INR data types

enum InrType {
  InrByte = 0,
  InrUnsignedInt,
  InrUnsignedLongInt,
  InrSignedInt,
  InrSignedLongInt,
  InrFloat,
  InrDouble
};
typedef enum InrType InrType;
  
extern char *InrTypeNames[];
extern int   InrTypeSizes[];
extern int   Inr2EnviTypes[];


struct InrGeorefInfo {
  double pixel_x;
  double pixel_y;
  double pixel_mapx;
  double pixel_mapy;
  double topleft_x;
  double topleft_y;
  double pixelsize_x;
  double pixelsize_y;
  int isUTM ;
  int UTMzone ;
  char UTMhemi[INRHDRBUFSIZE];
  char   datum[INRHDRBUFSIZE];
  char   projection[INRHDRBUFSIZE];
  char unit[INRHDRBUFSIZE];
};
typedef struct InrGeorefInfo InrGeorefInfo;

// ADDITION BY JPAUL APR 2006, 
// management of band names
struct InrBandNameInfo {
  int nBands ;
  char ** bandName ; 
};
typedef struct InrBandNameInfo InrBandNameInfo ;


// INR IO auxiliary methods
int    inrTypeIsInteger (InrType type);
int    inrGetType       (char *typeName, int typeSize, InrType *type);
double inrGetDataValue  (unsigned char *data, InrType type, int pos);
int    inrPutDataValue  (unsigned char *data, InrType type, int pos, double value);
int    inrReadHeader    (char *filename, int *size_x, int *size_y, int *size_z, InrType *type);
int    inrReadHeaderPtr (FILE *fpInr, int *size_x, int *size_y, int *size_z, InrType *type);
int    inrWriteHeader   (char *filename, int size_x, int size_y, int size_z, InrType type);
int    inrWriteHeaderPtr(FILE *fpInr, int size_x, int size_y, int size_z, InrType type);
int    inrCheckFile     (char *filename);
int    inrCheckFilePtr  (FILE *fp);


// INR IO major methods
unsigned char *inrReadDataSequence  (char *filename, int *size_x, int *size_y, int *size_z, InrType *type);
unsigned char *inrReadByteSequence  (char *filename, int *size_x, int *size_y, int *size_z);
float         *inrReadFloatSequence (char *filename, int *size_x, int *size_y, int *size_z);
int            inrWriteDataSequence (char *filename, int size_x, int size_y, int size_z, unsigned char *data, InrType type);
int            inrWriteByteSequence (char *filename, int size_x, int size_y, int size_z, unsigned char *data);
int            inrWriteFloatSequence(char *filename, int size_x, int size_y, int size_z, float *data);
int            inrBuildDataSequence (char *filelist, char *outname);
int            inrResetData         (char *filename, double value);

unsigned char *inrReadByteImage  (char *filename, int *size_x, int *size_y);
float         *inrReadFloatImage (char *filename, int *size_x, int *size_y);
int            inrWriteByteImage (char *filename, int size_x, int size_y, unsigned char *data);
int            inrWriteFloatImage(char *filename, int size_x, int size_y, float *data);

int            inrReadPoint (char *filename, int x, int y, unsigned char *data);
int            inrWritePoint(char *filename, int x, int y, unsigned char *data);
int            inrReadLine  (char *filename, int y, unsigned char *data);
int            inrWriteLine (char *filename, int y, unsigned char *data);
int            inrReadFrame (char *filename, int z, unsigned char *data);
int            inrWriteFrame(char *filename, int z, unsigned char *data);


// INR IO ENVI .hdr methods
char *inrBuildEnviHdr(char *filename, char *strHdr);
int   inrWriteEnviHdr(char *filename);
int   inrWriteGeoref (char *filename, InrGeorefInfo georef);
int   inrReadGeoref  (char *filename, InrGeorefInfo *georef);
int   inrVerifGeorefInfo (InrGeorefInfo georef1, InrGeorefInfo georef2);
InrBandNameInfo * getBandNameCFS1();
InrBandNameInfo * inrReadBandName(char *filename );
int   inrAppendBandName( char *filename, InrBandNameInfo *bn );
void  printBandName( InrBandNameInfo * bn ) ;


//int inrWriteColor (char *filename,int nClass);

// NOAA IO compatibility definitions
#define read_noaa_image       inrReadByteImage
#define write_noaa_image      inrWriteByteImage
#define read_noaa_sequence    inrReadByteSequence
#define write_noaa_sequence   inrWriteByteSequence
#define read_short_sequence   inrReadShortSequence
#define write_short_sequence  inrWriteShortSequence
#define read_indice_image     inrReadFloatImage
#define write_indice_image    inrWriteFloatImage
#define read_indice_sequence  inrReadFloatSequence
#define write_indice_sequence inrWriteFloatSequence
#define read_class_image      inrReadByteImage
#define write_class_image     inrWriteByteImage



#endif

