#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inrimage/image.h>
#include "file_tools.h"
#include "inr_io.h"

#define FNAMESIZE    1024


static char Ucmd[] =
"-ifile image -mult value -ofile image_multiplied";

static char Udetail[] =
"\nMultiplies an INRIMAGE image by a scalar value\n";




int main(int argc, char *argv[])
{
  char     fnameIn[FNAMESIZE];
  char     fnameOut[FNAMESIZE];

  unsigned char *data_in;
  float         *data_out;
  InrType type;
  int typesize;
  int size_x, size_y, size_z;
  int z;
  int i;
  double val;
  double mult;
  
  InrGeorefInfo georef;
  

  // initializes inrimage
  inr_init (argc, argv, "1.0", Ucmd, Udetail);

  // checks arguments
  if (!igetopt1 ("-ifile", "%s", fnameIn))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile" , "%s", fnameOut))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-mult", "%lf", &mult))
    iusage_ (Ucmd, Udetail);
  
  if (tstopts ())
    iusage_ (Ucmd, Udetail);

  if (!inrReadHeader(fnameIn, &size_x, &size_y, &size_z, &type))
  {
    fprintf(stderr, "Error: could not read header for image '%s'.\n", fnameIn);
    return 0;
  }

  typesize = InrTypeSizes[type]/8;
  data_in  = (unsigned char *)malloc(size_x*size_y*typesize*sizeof(unsigned char));    
  data_out = (float *)        malloc(size_x*size_y*sizeof(float));
  if (!data_in || !data_out)
  {
    fprintf(stderr, "Error: could not allocate memory.\n");
    return 0;
  }
  
  if (!inrWriteHeader(fnameOut, size_x, size_y, size_z, InrFloat))
  {
    fprintf(stderr, "Error: could write to output file '%s'.\n", fnameOut);
    return 0;
  }
  
  for (z=0; z<size_z; z++)
  {
    fprintf(stderr, "Reading layer %d of %d...\r", z+1, size_z);
    if (!inrReadFrame(fnameIn, z, data_in))
    {
      fprintf(stderr, "Error: could not read frame %d for image '%s'.\n", z, fnameIn);
      free(data_in);
      free(data_out);
      return 0;
    }
    fprintf(stderr, "Processing layer %d of %d...\r", z+1, size_z);
    
    for (i=0; i<size_x*size_y; i++)
    {
      val = inrGetDataValue(data_in, type, i);
      
      data_out[i] = val * mult;
    }
    
    if (!inrWriteFrame(fnameOut, z, (unsigned char *)data_out))
    {
      fprintf(stderr, "Error: could not write frame %d for output image.\n", z);
      free(data_in);
      free(data_out);
      return 0;
    }
  }
  
  if (inrReadGeoref(fnameIn, &georef))
    inrWriteGeoref(fnameOut, georef);
  else
    inrWriteEnviHdr(fnameOut);
    
  free(data_in);
  free(data_out);
  printf("INRIMAGE image successfully multiplied.\n");

  return 1;
}

