#ifndef __INR_IO_H__
#define __INR_IO_H__

#include "str_tools.h"
#define INRHDRBUFSIZE 4096


// INR data types

enum InrType {
  InrByte = 0,
  InrUnsignedInt,
  InrUnsignedLongInt,
  InrSignedInt,
  InrSignedLongInt,
  InrFloat,
  InrDouble
};
typedef enum InrType InrType;
  
extern char *InrTypeNames[];
extern int   InrTypeSizes[];
extern int   Inr2EnviTypes[];


struct InrGeorefInfo {
  double topleft_x;
  double topleft_y;
  double pixelsize_x;
  double pixelsize_y;
  char   datum[INRHDRBUFSIZE];
  char   projection[INRHDRBUFSIZE];
};
typedef struct InrGeorefInfo InrGeorefInfo;



// INR IO auxiliary methods
int inrGetType       (char *typeName, int typeSize, InrType *type);
int inrReadHeader    (char *filename, int *size_x, int *size_y, int *size_z, InrType *type);
int inrReadHeaderPtr (FILE *fpInr, int *size_x, int *size_y, int *size_z, InrType *type);
int inrWriteHeader   (char *filename, int size_x, int size_y, int size_z, InrType type);
int inrWriteHeaderPtr(FILE *fpInr, int size_x, int size_y, int size_z, InrType type);
int inrCheckFile     (char *filename);
int inrCheckFilePtr  (FILE *fp);


// INR IO major methods
unsigned char *inrReadDataSequence  (char *filename, int *size_x, int *size_y, int *size_z, InrType *type);
unsigned char *inrReadByteSequence  (char *filename, int *size_x, int *size_y, int *size_z);
float         *inrReadFloatSequence (char *filename, int *size_x, int *size_y, int *size_z);
int            inrWriteDataSequence (char *filename, int size_x, int size_y, int size_z, unsigned char *data, InrType type);
int            inrWriteByteSequence (char *filename, int size_x, int size_y, int size_z, unsigned char *data);
int            inrWriteFloatSequence(char *filename, int size_x, int size_y, int size_z, float *data);
int            inrBuildDataSequence (strList *filelist, char *outname);

unsigned char *inrReadByteImage  (char *filename, int *size_x, int *size_y);
float         *inrReadFloatImage (char *filename, int *size_x, int *size_y);
int            inrWriteByteImage (char *filename, int size_x, int size_y, unsigned char *data);
int            inrWriteFloatImage(char *filename, int size_x, int size_y, float *data);


// INR IO ENVI .hdr methods
char *inrBuildEnviHdr(char *filename, char *strHdr);
int   inrWriteEnviHdr(char *filename);
int   inrWriteGeoref (char *filename, InrGeorefInfo georef);
int   inrReadGeoref  (char *filename, InrGeorefInfo *georef);



// NOAA IO compatibility definitions
#define read_noaa_image       inrReadByteImage
#define write_noaa_image      inrWriteByteImage
#define read_noaa_sequence    inrReadByteSequence
#define write_noaa_sequence   inrWriteByteSequence
#define read_short_sequence   inrReadShortSequence
#define write_short_sequence  inrWriteShortSequence
#define read_indice_image     inrReadFloatImage
#define write_indice_image    inrWriteFloatImage
#define read_indice_sequence  inrReadFloatSequence
#define write_indice_sequence inrWriteFloatSequence
#define read_class_image      inrReadByteImage
#define write_class_image     inrWriteByteImage



#endif

