#include <stdio.h>
#include <stdlib.h>
#include <inrimage/image.h>
#include "inr_io.h"


#define FNAMESIZE  1024
#define MIN(x,y) ((x<y)?x:y)
#define MAX(x,y) ((x>y)?x:y)



static char Ucmd[] = "-ifile image -set new_values_file -ofile new_image\n";

static char Udetail[] =
"Defines new values at specified (x,y) coordinates for an INRIMAGE image file.\n\
\nThe file defining the new values should be a text file containing the\n\
following information at each line:\n\
\t'x  y  new_value'\n\
indicating that the pixel at position (x,y) should belong be new_value\n\
\nInput:\n\
\t-ifile\tFile name of the input image\n\
\t-set\tFile name of the text file defining the new values\n\
\nOutput:\n\
\t-ofile\tFile name of the output new image\n";





int main (int argc, char **argv)
{
  char fnameInput[FNAMESIZE];
  char fnameOutput[FNAMESIZE];
  char fnameSet[FNAMESIZE];
  InrGeorefInfo georef;
  InrType       type;
  int           typesize;

  FILE *fpSet;

  unsigned char *data;

  int size_x, size_y, size_z;
  int x, y;
  int pos;
  double new_value;
  

  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-set"  , "%s", fnameSet))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile", "%s", fnameOutput))
    iusage_ (Ucmd, Udetail);
  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);


  // reads header information for all input data and checks if they are appropriate
  if (!inrReadHeader(fnameInput, &size_x, &size_y, &size_z, &type))
  {
    fprintf(stderr, "Error: could not read header information for input file '%s'.\n", fnameInput);
    return 0;
  }
  if (size_z != 1)
  {
    fprintf(stderr, "Error: input image '%s' should have only one layer.\n", fnameInput);
    return 0;
  }
  fpSet = fopen(fnameSet, "rt");
  if (!fpSet)
  {
    fprintf(stderr, "Error: could not text file '%s'.\n", fnameSet);
    return 0;
  }
 
  // reads the input classification image
  data = inrReadDataSequence(fnameInput, &size_x, &size_y, &size_z, &type);
  if (!data)
  {
    fprintf (stderr, "Error: could not read input file '%s'.\n", fnameInput);
    fclose(fpSet);
    return 0;
  }
  typesize = InrTypeSizes[type]/8;
  

  // processes the data
  while (!feof(fpSet))
  {
    int nitems = fscanf(fpSet, "%d %d %lf", &x, &y, &new_value);
    if (nitems != 3)
      continue;
    if (x<0 || x>=size_x || y<0 || y>size_y)
      continue;

    pos = x + y*size_x;
    
    inrPutDataValue(data, type, pos, new_value);
  }
  fclose(fpSet);

  
  // writes output translated classification
  if (!inrWriteDataSequence(fnameOutput, size_x, size_y, 1, data, type))
  {
    fprintf (stderr, "Error: could not write to output file '%s'.\n", fnameOutput);
    free(data);
    return 0;
  }
  free(data);
  
  // copies georreference info if present
  if (inrReadGeoref(fnameInput, &georef))
    inrWriteGeoref(fnameOutput, georef);
  
  fprintf(stderr, "Image pixels successfully set.\n");
  return 1;
}
