/* 
   biblio d'algebre lineaire
   Antoine BOURDAIS
   Jean-Paul BERROIR
   INRIA 1999-2003
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <memory.h>
#include "linearalgebra.h"


/* valeur absolue */
double ValeurAbs(double a)
{
  if (a < 0.0)
    return -a;
  else
    return a;
}

/* nombre alatoire entre -5 et 5 */
double Alea()
{
  //*** MILTON ***
  #if (defined(WIN32) || defined(_WIN32))
  return 10.0*((double)rand()/RAND_MAX - 0.5);
  #else
  return (double)10.0*(drand48()-0.5);
  #endif
}

/* liberation d'une matrice */
void free_matrice( matrice **m )
{
  free ( (*m)->val ) ;
  (*m)->val = NULL ;
  free( *m ) ;
  (*m)  = NULL ;

}


/* copie conforme d'une matrice */
void Copier_Matrice( matrice *dest , matrice *src)
{
  dest->nbColonnes = src->nbColonnes ;
  dest->nbLignes = src->nbLignes ;
  memcpy( dest->val , src->val , src->nbColonnes *  src->nbLignes * sizeof(double) ) ;
}

/* initialisation d'une matrice nxn de coefs alatoires */
matrice *Rand_Matrice(int n)
{
  int i;
  matrice *m ;

  m=(matrice*)malloc(sizeof(matrice));
  m->nbColonnes = m->nbLignes = n ;
  m->val=(double*)malloc(n*n*sizeof(double));
  for (i=0;i<n*n;i++) m->val[i] = Alea() ;

  return m;
}

/* initialisation d'une atrice alatoire symtrique */
matrice * Rand_Sym_Matrice(int n)
{
  matrice *m1,*m2,*m ;
  int i;

  m1 = Rand_Matrice(n);
  m2 = Transposer_Matrice(m1);
  m=Additionner_Matrice(m1,m2);
  free_matrice(&m1),free_matrice(&m2);
  for(i=0;i<n;i++) m->val[i+n*i] = abs(m->val[i+n*i]) ;
  return m;
}
/* initialisation d'un vecteur aleatoire */
matrice * Rand_Vector( int n )
{
  matrice * m ;
  int i ;
  m=(matrice*)malloc(sizeof(matrice));
  m->nbColonnes = n;
  m->nbLignes = 1 ;
   m->val=(double*)malloc(n*sizeof(double));
   for (i=0;i<n;i++) m->val[i] = Alea() ;
   return m;
}
/* initialisation interactive d'une matrice */
matrice* Initialiser_Matrice()
{
  int i;
  int j;
  int nbLignes=0;
  int nbColonnes=0;
  matrice* mat;

  mat = (matrice*)malloc(sizeof(matrice));

  do
    {
      printf("Nombre de lignes:");
      scanf("%d",&nbLignes);
      if (nbLignes>=10) printf("Entrer un nombre de lignes < 10 !");
    }
  while (nbLignes>=10);

  do
    {
      printf("Nombre de colonnes:");
      scanf("%d",&nbColonnes);
      if (nbColonnes>=10) printf("Entrer un nombre de colonnes < 10 !");
    }
  while (nbColonnes>=10);

  mat->nbLignes = nbLignes;
  mat->nbColonnes = nbColonnes;
  mat->val = (double*)calloc(nbLignes*nbColonnes,sizeof(double));

  printf("Initialisation de la matrice\n\n");

  for(i=0; i<mat->nbLignes; i++)
    {
      for(j=0; j<mat->nbColonnes; j++)
	{
	  printf("valeur de mat[%d][%d]:",i+1,j+1);
	  scanf("%lf",&mat->val[nbColonnes*i+j]);
	}
    }


  return mat;
  free(mat->val);
  free(mat);
}

/* affichage  l'cran d'une matrice */
void Afficher_Matrice(matrice* m)
{
  int i;
  int j;

  for (i=0; i<m->nbLignes; i++)
    {
      printf("[");
      for (j=0; j<m->nbColonnes; j++)
        {
          if(j==0) printf("%10.6f",m->val[m->nbColonnes*i+j]);
          else printf(",%10.6f",m->val[m->nbColonnes*i+j]);
        }
      printf("]\n");
    }

}
/* criture dans un fichier d'une matrice */
void Printf_Matrice(FILE *f,matrice* m)
{
  int i;
  int j;

  for (i=0; i<m->nbLignes; i++)
    {
      fprintf(f,"[");
      for (j=0; j<m->nbColonnes; j++)
        {
          if(j==0) fprintf(f,"%f",m->val[m->nbColonnes*i+j]);
          else fprintf(f,",%f",m->val[m->nbColonnes*i+j]);
        }
      fprintf(f,"]\n");
    }

}

/* transposition */
matrice* Transposer_Matrice(matrice* m)
{
  int i;
  int j;
  float val;
  matrice* transpose;

  transpose = (matrice*)malloc(sizeof(matrice));

  transpose->nbLignes = m->nbColonnes;
  transpose->nbColonnes = m->nbLignes;
  transpose->val = (double*)malloc(m->nbLignes * m->nbColonnes * sizeof(double));
 

  for(j=0; j<transpose->nbLignes; j++)
    {
      for(i=0; i<transpose->nbColonnes; i++)
        {
          val = m->val[m->nbColonnes*i+j];
          transpose->val[transpose->nbColonnes*j+i] = val;
        }
    }
  return transpose;
  free(transpose);
  free(transpose->val);
}

/* inversion */
matrice* Inverser_Matrice(matrice* m)
{
  int *index;
  int i,j,n;
  matrice *inverse, *temp;
  double parite;
  double *col;  

  if (m->nbColonnes != m->nbLignes)
    {
      fprintf(stderr,"Probleme d'inversion :La matrice n'est pas carree!\n");
      exit(1);
    }
  
  n = m->nbColonnes;
  inverse = (matrice*)malloc(sizeof(matrice));
  temp = (matrice*)malloc(sizeof(matrice));
  col = (double*)malloc(n * sizeof(double));
  index = (int*)malloc(n * sizeof(int));

  inverse->nbLignes = n;
  inverse->nbColonnes = n;
  inverse->val = (double*)malloc(n * n * sizeof(double));
  temp->nbLignes = n;
  temp->nbColonnes = n;
  temp->val = (double*)malloc(n * n * sizeof(double));

  for(i=0; i<n; i++)
    for(j=0; j<n; j++)
      temp->val[n*i+j] = m->val[n*i+j];

  DecompositionLU(temp, index, &parite);
  for(j=0; j<n; j++)
    {
      for(i=0; i<n; i++)
        col[i] = 0.0;
      col[j] = 1.0;

      TraitementLU(temp, index, col);
      for(i=0; i<n; i++)
        inverse->val[n*i+j] = col[i];
    }
  free(temp->val);
  free(temp);
  free(col);
  free(index);
  return inverse;


}

/* dcomposition LU */
void DecompositionLU(matrice* m, int* index, double* parite)
{
  int i,j,k,n;
  int imax;
  double pivot, somme, div, temp;
  double* coeff;

  n = m->nbColonnes;
  coeff = (double*)malloc(n*sizeof(double));
  *parite = 1.0;

  for(i=0; i<n; i++)
    {
      pivot = 0.0;
      for(j=0; j<n; j++)
        if ((temp = ValeurAbs(m->val[n*i+j])) > pivot)
          pivot = temp;
      if (pivot == 0.0)
        fprintf(stderr,"Probleme : Matrice singuliere!\n");
    
       coeff[i] = 1.0/pivot;
    }

  for(j=0; j<n; j++)
    {


      for(i=0; i<j; i++)
        {
          somme = m->val[n*i+j];          
          for(k=0; k<i; k++)
            somme -= m->val[n*i+k] * m->val[n*k+j];
          m->val[n*i+j] = somme;
        }
      
      pivot = 0.0;

      for(i=j; i<n; i++)
        {
          somme = m->val[n*i+j];
          for(k=0; k<j; k++)
            somme -= m->val[n*i+k] * m->val[n*k+j];
          m->val[n*i+j] = somme;
          if ((div = coeff[i]*ValeurAbs(somme)) >= pivot)
            {
              pivot = div;
              imax = i;
            }
        }
      if (j != imax)
        {
          for(k=0; k<n; k++)
            {
              div = m->val[n*imax+k];
              m->val[n*imax+k] = m->val[n*j+k];
              m->val[n*j+k] = div;
            }
          *parite = -(*parite);
          coeff[imax] = coeff[j];
        }     
      index[j] = imax;
      if (m->val[n*j+j] == 0.0) 
        m->val[n*j+j] = Epsilon;
      
      if (j != n-1)
        {
          div = 1.0/m->val[n*j+j];
          for(i=j+1; i<n; i++)
            m->val[n*i+j] = m->val[n*i+j] * div;
        }

    }
free(coeff);
}          


/* utilisation de la dcomposition LU */
void TraitementLU(matrice *m, int *index, double *b)
{
  int i, j, numIndex, ii;
  double somme;
  int n;
  ii = -1;
  n = m->nbColonnes;

  for(i=0; i<n; i++)
    { 
      numIndex = index[i];
      somme = b[numIndex];
      b[numIndex] = b[i];
      if (ii+1 != 0)
        for(j=ii; j<=i-1; j++)
          somme -=  m->val[n*i+j] * b[j];
      else if (somme != 0)
        ii = i;
      b[i] = somme;
    }

  for(i=n-1; i>=0; i--)
    {
      somme = b[i];
      for(j=i+1; j<n; j++)
        somme -=  m->val[n*i+j] * b[j];
       b[i] = somme/m->val[n*i+i];
    }
}

/* produit matriciel */
matrice* Multiplier_Matrice(matrice* a, matrice* b)
{
  int i, j, k;
  matrice* produit;
  double somme;

  if(a->nbColonnes != b->nbLignes)
    {
      fprintf(stderr,"Les dimensions de ces matrices sont incompatibles op: * !\n");
      exit(1);
    }

  //*** MILTON ***
  produit = (matrice*)malloc(sizeof(matrice));
  produit->val = (double*)malloc(a->nbLignes * b->nbColonnes * sizeof(double));
  //  produit->val = (double*)malloc(a->nbColonnes * a->nbColonnes * sizeof(double));
  produit->nbLignes = a->nbLignes;
  produit->nbColonnes = b->nbColonnes;
  
  for(i=0; i<produit->nbLignes; i++)
    {
      for(j=0; j<produit->nbColonnes; j++)
        {
          somme = 0.0;
          for(k=0; k<a->nbColonnes; k++)
            {
              if(a->val[a->nbColonnes*i+k]!=0.0 && b->val[b->nbColonnes*k+j]!=0.0)
              somme += (double) ((a->val[a->nbColonnes*i+k]) * (b->val[b->nbColonnes*k+j]));
            }
          produit->val[produit->nbColonnes*i+j] = somme;
        }
    }
  return produit;
}


/* dterminant */
double Determinant(matrice* m)
{
  int *index;
  int i,j,n;
  matrice *temp;
  double d;
  double *col;  

  if (m->nbColonnes != m->nbLignes)
    {
      fprintf(stderr,"Probleme determinant : la matrice n'est pas carree\n");
      exit(1);
    }
  
  n = m->nbColonnes;
  temp = (matrice*)malloc(sizeof(matrice));
  col = (double*) malloc (n*sizeof(double));
  index = (int*) malloc (n*sizeof(int));

  temp->nbLignes = n;
  temp->nbColonnes = n;
  temp->val = (double*)malloc(n * n * sizeof(double));

  for(i=0; i<n; i++)
    for(j=0; j<n; j++)
      temp->val[n*i+j] = m->val[n*i+j];

  DecompositionLU(temp, index, &d);
  for(j=0; j<n; j++)
    {
    d *= temp->val[n*j+j];
    }
free(temp->val);
free(temp);
free(col);
free(index);
  return d;
  free(temp->val);
  free(temp);
}

/* produit des elements d'une matrice */
double DeterminantCarre(matrice* m)
{
  int i;
  double temp=1.0;
  for(i=0;i<m->nbLignes;i++)
    temp*=m->val[i+i*m->nbColonnes];
  return temp;
}

/* addition matricielle */
matrice* Additionner_Matrice(matrice* a, matrice* b)
{
  int i, j;
  matrice* somme;
  somme = (matrice*)malloc(sizeof(matrice));
  

  if((a->nbLignes != b-> nbLignes) || (a->nbColonnes != b->nbColonnes))
    {
      fprintf(stderr,"Les dimensions de ces matrices sont incompatibles op +!\n");
      exit(1);
    }

  somme->val = (double*)malloc(a->nbLignes * a->nbColonnes * sizeof(double));
  somme->nbLignes = a->nbLignes;
  somme->nbColonnes = a->nbColonnes;
  
  for(i=0; i<somme->nbLignes; i++)
    {
      for(j=0; j<somme->nbColonnes; j++)
        {
          somme->val[somme->nbColonnes*i+j] = a->val[a->nbColonnes*i+j]+b->val[b->nbColonnes*i+j];
        }
    }
  return somme;
  free(somme->val);
  free(somme);
}


/* soustraction matricielle */
matrice* Soustraire_Matrice(matrice* a, matrice* b)
{
  int i, j;
  matrice* difference;

  difference = (matrice*)malloc(sizeof(matrice));
  difference->nbLignes = a->nbLignes;
  difference->nbColonnes = a->nbColonnes; 
  difference->val = (double*) malloc (a->nbLignes * a->nbColonnes * sizeof(double));

  if((a->nbLignes != b->nbLignes) || (a->nbColonnes != b->nbColonnes))
    {
      fprintf(stderr,"Les dimensions de ces matrices sont incompatibles op - !\n");
      exit(1);
    }
  
  for(i=0; i<difference->nbLignes; i++)
    {
      for(j=0; j<difference->nbColonnes; j++)
        {
          difference->val[difference->nbColonnes*i+j] = (double)(a->val[a->nbColonnes*i+j] - b->val[b->nbColonnes*i+j]);
        }
    }
  return difference;

  free(difference->val);
  free(difference);
}


/* mise en valeur abs de tous les elements d'une matrice */
matrice* Module_Matrice(matrice* m)
{
  int i, j;
  for(i=0; i<m->nbLignes; i++)
    {
      for(j=0; j<m->nbColonnes; j++)
        {
          m->val[m->nbColonnes*i+j] = ValeurAbs(m->val[m->nbColonnes*i+j]);
        }
    }
  return m;
}


/* cration d'une matrice nulle */
matrice* Matrice_Nulle(int nbLignes, int nbColonnes)
{
  matrice* nul;
  int i, j;

  nul = (matrice*)malloc(sizeof(matrice));
  nul->nbLignes = nbLignes;
  nul->nbColonnes = nbColonnes;
  nul->val = (double*) malloc (nbLignes*nbColonnes*sizeof(double));
  for(i=0; i<nbLignes; i++)
    {
      for(j=0; j<nbColonnes; j++)
        {
          nul->val[nbColonnes*i+j] = 0.0;
        }
    }

  return nul;
}

/* cration d'une matrice identit */
void Matrice_Identite(matrice* id, int nbLignes, int nbColonnes)
{
  int i, j;
  
  for(i=0; i<nbLignes; i++)
    {
      for(j=0; j<nbColonnes; j++)
        {
          if(i==j)
            id->val[nbColonnes*i+j] = 1.0;
          else
            id->val[nbColonnes*i+j] = 0.0;
        }
    }

  //return id;
}

/* addition matricielle, rsultat dans premier oprande */
void Ajouter_Matrice(matrice* a, matrice* b)
{
  int i, j;
  
  if((a->nbLignes != b-> nbLignes) || (a->nbColonnes != b->nbColonnes))
    {
      fprintf(stderr,"Les dimensions de ces matrices sont incompatibles op +!\n");
      exit(1);
    }

  for(i=0; i<a->nbLignes; i++)
    {
      for(j=0; j<a->nbColonnes; j++)
        {
          a->val[a->nbColonnes*i+j] = a->val[a->nbColonnes*i+j]+b->val[b->nbColonnes*i+j];
        }
    }
}

/* transposee x matrice */
matrice* Calculer_XtX(matrice* a)
{
  matrice* mat;
  int i, j;

  mat = (matrice*) malloc (sizeof(matrice));
  mat->nbLignes = a->nbColonnes;
  mat->nbColonnes = a->nbColonnes;
  mat->val = (double*) malloc (a->nbColonnes*a->nbColonnes*sizeof(double));
  for(i=0; i<a->nbColonnes; i++)
    {
      for(j=0; j<a->nbColonnes; j++)
        {
          mat->val[mat->nbColonnes*i+j] = a->val[i]*a->val[j];
        }
    }

  return mat;
}

/* multiplication par un scalaire */
void Multiplication_Scalaire_Matrice(double scalaire, matrice* a)
{
  int i, j;
  
  for(i=0; i<a->nbLignes; i++)
    {
      for(j=0; j<a->nbColonnes; j++)
        {
          a->val[a->nbColonnes*i+j] = a->val[a->nbColonnes*i+j] * scalaire;
        }
    }
}

/* jacobien */
void Jacobi(matrice* a, int n, double* d, matrice* v, int *nrot)
{
  int j, iq, ip, i;
  double tresh, theta, tau, t, sm, s, h, g, c;
  matrice *b, *z;
  int nbLignes, nbColonnes;

  nbLignes = a->nbLignes;
  nbColonnes = a->nbColonnes;

  b = (matrice*) malloc (sizeof(matrice));
  z = (matrice*) malloc (sizeof(matrice));

  b->nbLignes = 1;
  b->nbColonnes = n;
  b->val = (double*) malloc (n*sizeof(double));
  z->nbLignes = 1;
  z->nbColonnes = n;
  z->val = (double*) malloc (n*sizeof(double));

  Matrice_Identite(v, v->nbLignes, v->nbColonnes);

  for(ip=1; ip<=n; ip++)
    {
      b->val[(ip-1)] = a->val[(ip-1)+(ip-1)*nbColonnes];
      d[(ip-1)] = a->val[(ip-1)+(ip-1)*nbColonnes];
      z->val[(ip-1)] = 0.0;
    }

  *nrot = 0;

  for(i=1; i<=50; i++)
    {
      sm = 0.0;
      for(ip=1; ip<n; ip++)
        {
          for(iq = ip+1; iq<=n; iq++)
            sm += ValeurAbs(a->val[(ip-1)+nbColonnes*(iq-1)]);
        }  
      if(sm==0.0)
        {
          free(b->val);
          free(b);
          free(z->val);
          free(z);
          return;
        }
      if(i<4)
        tresh = 0.2*sm/(n*n);
      else
        tresh = 0.0;    
      for(ip=1; ip<=n-1; ip++)
        {
          for(iq=ip+1; iq<=n; iq++)
            {
              g = 100.0*ValeurAbs(a->val[(ip-1)+nbColonnes*(iq-1)]);
              if(i>4 && (ValeurAbs(d[(ip-1)])+g) == (ValeurAbs(d[(ip-1)])) && (ValeurAbs(d[(iq-1)])+g) == (ValeurAbs(d[(iq-1)])))
                a->val[(ip-1)+nbColonnes*(iq-1)] = 0.0;
              else
                if(ValeurAbs(a->val[(ip-1)+nbColonnes*(iq-1)])>tresh)
                  {
                    h = d[(iq-1)] - d[(ip-1)];
                    if((ValeurAbs(h)+g) == ValeurAbs(h))
                      t = (a->val[(ip-1)+nbColonnes*(iq-1)])/h;
                    else
                      {
                        theta = 0.5*h/(a->val[(ip-1)+nbColonnes*(iq-1)]);
                        t = 1.0/(ValeurAbs(theta)+sqrt(1.0+theta*theta));
                        if(theta < 0.0)
                          t = -t;
                      }

                    c = 1.0/sqrt(1+t*t);
                    s = t*c;
                    tau = s/(1.0+c);
                    h = t*a->val[(ip-1)+nbColonnes*(iq-1)];
                    z->val[(ip-1)] -= h;
                    z->val[(iq-1)] += h;
                    d[(ip-1)] -= h;
                    d[(iq-1)] += h;
                    a->val[(ip-1)+nbColonnes*(iq-1)] = 0.0;
                    for(j=1; j<=ip-1; j++)
                      Rotate(a, j-1, ip-1, j-1, iq-1, g, h, s, tau);
                    for(j=ip+1; j<=iq-1; j++)
                      Rotate(a, ip-1, j-1, j-1, iq-1, g, h, s, tau);
                    for(j=iq+1; j<=n; j++)
                      Rotate(a, ip-1, j-1, iq-1, j-1, g, h, s, tau);
                    for(j=1; j<=n; j++)
                      Rotate(v, j-1, ip-1, j-1, iq-1, g, h, s, tau);
                    ++(*nrot);
                  }
            }
        }
      for(ip=1; ip<=n; ip++)
        {
          b->val[ip-1] += z->val[ip-1];
          d[ip-1] = b->val[ip-1];
          z->val[ip-1] = 0.0;
        }
    }
  printf("Too many iterations in routine jacobi(file matrice.c");
}

/* permuation de termes d'une matrice */
void Rotate(matrice* a, int i, int j, int k, int l, double g, double h, double s, double tau)
{
  g = a->val[i+a->nbColonnes*j];
  h = a->val[k+a->nbColonnes*l];
  a->val[i+a->nbColonnes*j] = g-s*(h+g*tau);
  a->val[k+a->nbColonnes*l] = h+s*(g-h*tau);
}

/* pour systeme propre */
void eigrst(double* d, matrice* v, int n)
{
  int k, j, i;
  double p;

  for(i=0; i<n-1; i++)
    {
      k = i;
      p = d[i];
      for(j=i+1;j<n;j++)
        if(d[j]>=p)
          {
            k = j;
            p = d[j];
          }
      if(k!=(i))
        {
          d[k] = d[i];
          d[i] = p;
          for(j=0; j<n; j++)
            {
              p = v->val[j+i*v->nbColonnes];
              v->val[j+i*v->nbColonnes] = v->val[j+k*v->nbColonnes];
              v->val[j+k*v->nbColonnes] = p;
            }
        }
    }
}

/* vecteurs propres */
/*a is a diagonal matrix*/
void EigenValuesVectors(matrice* a, int n, double* d, matrice* v)
{
  int nrot;
  
  Jacobi(a, n, d, v, &nrot);
  eigrst(d, v, n);
}
