/***************************************************************************
                          profile_feature.h  -  description
                             -------------------
    Modified             : September, 2007
    By                   : (C) 2007 by Carlos de la Torre
    Email                : Carlos.de_la_Torre@inria.fr
***************************************************************************/

#ifndef __PROFILE_FEATURES_H__
#define __PROFILE_FEATURES_H__

#include "inr_io.h"

#define PROF_MAX_MODES                 10
#define PROF_MODE_HEIGHT_MIN           0.3
#define PROF_MODE_HEIGHT_CERTAIN       0.7
#define PROF_MODE_HEIGHT_DOUBT         0.4
#define PROF_MODE_DURATION_DOUBT       200
#define PROF_MODE_DURATION_CERTAIN_NOT 300


/*
enum profModeFeaturesIndex {
  profMode_datePeak = 0,
  profMode_dateWidthPeak,
  profMode_dateWidthLeft,
  profMode_dateWidthRight,
  profMode_valuePeak,
  profMode_valueDeltaLeft,
  profMode_valueDeltaRight,
  profMode_integral,
  profMode_slopeLeft,
  profMode_slopeRight,
  PROF_NUM_FEATURES_MODE
};
*/

enum profFeaturesIndex {
  prof_mean = 0,
  prof_min,
  prof_max,
  prof_maxDate,
  prof_begin,
  prof_deltaMaxMin,
  prof_deltaBeginEnd,
  prof_delta45days,
  prof_slopeMin,
  prof_slopeAbsMean,
  prof_valueStddev,
  prof_valueSlopeMin,
  prof_valueSlopeAbsMean,
  prof_valueInterannualLoss,
  prof_premaxMean,
  prof_premaxMin,
  prof_premaxDeltaMaxMin,
  prof_premaxSlopeAbsMean,
  PROF_NUM_FEATURES
};


extern char *profFeaturesNames[];


typedef struct profMode
{
  double datePeak;       // Date of the peak
  double dateWidthPeak;  // Duration (or width) of the mode
  double dateWidthLeft;  // Duration of the rising of the mode
  double dateWidthRight; // Duration of the falling of the mode
  double valuePeak;      // NDVI value at the peak
  double valueDeltaLeft; // Amplitude (height) from the left
  double valueDeltaRight;// Amplitude (height) from the right
  double integral;       // Area under the mode
  double slopeLeft;      // Time (in days) to ascend from half the slope to the peak
  double slopeRight;     // Time (in days) to descend from the peak to half the slope
  int    nSamples;       // No idea... Old feature... we'll see...
} profMode;

typedef struct profFeatures
{
  double   val[PROF_NUM_FEATURES];
  double   nModesStats;
  int      nModes;
  profMode modes[PROF_MAX_MODES];
} profFeatures;


// initialization
int profInitStruct (profFeatures *features);

// operations
double *profPolyDerivate   (double *coef, int degree, double *output);
double  profPolyValue      (double *coef, int degree, double point);
int     profFeaturesSum    (profFeatures *sum, profFeatures *sum2, profFeatures term);
int     profFeaturesSquare (profFeatures term1, profFeatures *result);
int     profFeaturesMean   (profFeatures sum, int n, profFeatures *result);
int     profFeaturesStddev (profFeatures sum, profFeatures sum2, int n, profFeatures *result);

// methods for computing and manipulating modes
int profModesAdd     (profMode mode1, profMode mode2, profMode *result);
int profModesSquare  (profMode mode1, profMode *result);
int profModesMean    (profMode sum, profMode *result);
int profModesStddev  (profMode sum, profMode sum2, profMode *result);

int profModeFind     (profFeatures features, profMode mode);
int profModesMatch   (profFeatures features1, profFeatures features2, int *result);

int profDefineMode (double *coef, int degree, double *profile, double *profile_deriv,
                    double  dateBegin, double dateEnd, double dateInc,
                    int iBegin, int iPeak, int iEnd, float slopeLeft, float slopeRight ,profFeatures *features);

int profComputeModes (double *coef, int degree, double *profile, double *profile_deriv,
                      double dateBegin, double dateEnd, double dateInc, 
                      profFeatures *features);

int profIsLocalMax  (int iDate, double *profile_deriv, int nDates);
int profIsLocalMin  (int iDate, double *profile_deriv, int nDates);
int profFindLeftMin (int iDate, double *profile_deriv, int nDates);
int profFindRightMin(int iDate, double *profile_deriv, int nDates);
float profSlope     (int iPeak,int iLow,double *profile, float ratio);
float profModeHeight(profMode mode);


// auxiliary methods for computing features
void profComputePolyFeatures_PreMax(double *profile, double *profile_deriv,
                                    double dateBegin, double dateEnd, double dateInc,
                                    profFeatures *features);

double profGetMean(int dateCenter, int dateWidth,
                   double *profile, int *dates, int nDates,
                   double *profilePrev, int *datesPrev, int nDatesPrev);
                         
double profGetPrevMean(int dateCenter, int dateWidth,
                       double *profile, int *dates, int nDates,
                       double *profilePrev, int *datesPrev, int nDatesPrev);

double profGetPrevMin(int dateCenter, int dateWidth,
                      double *profile, int *dates, int nDates,
                      double *profilePrev, int *datesPrev, int nDatesPrev);


// major methods
int profComputeFeatures(double *values, int *dates, int nDates,
                        double *valuesPrev, int *datesPrev, int nDatesPrev,
                        int degree, profFeatures *features);

int profComputePolyFeatures(double *coef, int degree,
                            double dateBegin, double dateEnd, double dateInc, 
                            profFeatures *features);

int profComputeValueFeatures(double *profile, int *dates, int nDates,
                             double *profilePrev, int *datesPrev, int nDatesPrev,
                             profFeatures *features);
                         
int    profFitPoly(double *values, int *dates, int nDates, double *coef, int degree);

double profGetPolyValue(double *coef, int degree, int date);

// methods providing feature names for user information
InrBandNameInfo * getBandNameCFS(int nFeatures);



#endif

