#ifndef __PROFILE_FEATURES_H__
#define __PROFILE_FEATURES_H__

#include "inr_io.h"

#define PROF_MAX_MODES              10
#define PROF_DEFAULT_MODE_TOLERANCE 0.3
#define PROF_PEAK_TOLERANCE         0.1


/*
enum profModeFeaturesIndex {
  profMode_datePeak = 0,
  profMode_dateWidthPeak,
  profMode_dateWidthLeft,
  profMode_dateWidthRight,
  profMode_valuePeak,
  profMode_valueDeltaLeft,
  profMode_valueDeltaRight,
  profMode_integral,
  profMode_slopeLeft,
  profMode_slopeRight,
  PROF_NUM_FEATURES_MODE
};
*/

enum profFeaturesIndex {
  prof_mean = 0,
  prof_min,
  prof_max,
  prof_maxDate,
  prof_begin,
  prof_deltaMaxMin,
  prof_deltaBeginEnd,
  prof_delta150days,
  prof_slopeMin,
  prof_slopeAbsMean,
  prof_valueStddev,
  prof_valueSlopeMin,
  prof_valueSlopeAbsMean,
  prof_valueInterannualLoss,
  prof_premaxMean,
  prof_premaxMin,
  prof_premaxDeltaMaxMin,
  prof_premaxSlopeAbsMean,
  PROF_NUM_FEATURES
};


extern char *profFeaturesNames[];

typedef struct profMode
{
  double datePeak;
  double dateWidthPeak;
  double dateWidthLeft;
  double dateWidthRight;
  double valuePeak;
  double valueDeltaLeft;
  double valueDeltaRight;
  double integral;
  double slopeLeft;
  double slopeRight;
  //double val[PROF_NUM_FEATURES_MODE];
  int    nSamples;
} profMode;

typedef struct profFeatures
{
  double   val[PROF_NUM_FEATURES];
  double   nModesStats;
  int      nModes;
  profMode modes[PROF_MAX_MODES];
} profFeatures;


// initialization
int profInitStruct (profFeatures *features);

// operations
double *profPolyDerivate   (double *coef, int degree, double *output);
double  profPolyValue      (double *coef, int degree, double point);
int     profFeaturesSum    (profFeatures *sum, profFeatures *sum2, profFeatures term);
int     profFeaturesSquare (profFeatures term1, profFeatures *result);
int     profFeaturesMean   (profFeatures sum, int n, profFeatures *result);
int     profFeaturesStddev (profFeatures sum, profFeatures sum2, int n, profFeatures *result);

// methods for computing and manipulating modes
int profModesAdd     (profMode mode1, profMode mode2, profMode *result);
int profModesSquare  (profMode mode1, profMode *result);
int profModesMean    (profMode sum, profMode *result);
int profModesStddev  (profMode sum, profMode sum2, profMode *result);

int profModeFind     (profFeatures features, profMode mode);
int profModesMatch   (profFeatures features1, profFeatures features2, int *result);

int profDefineMode (double *coef, int degree, double *profile, double *profile_deriv,
                    double  dateBegin, double dateEnd, double dateInc,
                    int iBegin, int iPeak, int iEnd, profFeatures *features);

int profComputeModes (double *coef, int degree, double *profile, double *profile_deriv,
                      double dateBegin, double dateEnd, double dateInc, 
                      profFeatures *features);


// auxiliary methods for computing features
void profComputePolyFeatures_PreMax(double *profile, double *profile_deriv,
                                    double dateBegin, double dateEnd, double dateInc,
                                    profFeatures *features);

double profGetMean(int dateCenter, int dateWidth,
                   double *profile, int *dates, int nDates,
                   double *profilePrev, int *datesPrev, int nDatesPrev);
                         
double profGetPrevMean(int dateCenter, int dateWidth,
                       double *profile, int *dates, int nDates,
                       double *profilePrev, int *datesPrev, int nDatesPrev);

double profGetPrevMin(int dateCenter, int dateWidth,
                      double *profile, int *dates, int nDates,
                      double *profilePrev, int *datesPrev, int nDatesPrev);


// major methods
int profComputeFeatures(double *values, int *dates, int nDates,
                        double *valuesPrev, int *datesPrev, int nDatesPrev,
                        int degree, profFeatures *features);

int profComputePolyFeatures(double *coef, int degree,
                            double dateBegin, double dateEnd, double dateInc, 
                            profFeatures *features);

int profComputeValueFeatures(double *profile, int *dates, int nDates,
                             double *profilePrev, int *datesPrev, int nDatesPrev,
                             profFeatures *features);
                         
int    profFitPoly(double *values, int *dates, int nDates, double *coef, int degree);

double profGetPolyValue(double *coef, int degree, int date);

InrBandNameInfo * getBandNameCFS(void);



#endif

