#ifndef __STD_H__
#define __STD_H__



/*
 * TRANSITION
 *
 * Definition of a single transition within a STD
 */
typedef struct stdTransition
{
    // index of the profile class of the transition
    int iProfile;

    // index of the consequent state given the profile class
    int iNextState;
}
stdTransition;



/*
 * STATE_TRANSITIONS
 *
 * Definition of a state, listing its transitions
 */
typedef struct stdState
{
    // array of transitions
    // there will potentially be one transition for each profile class
    stdTransition *transitions;

    // number of transitions registered for the state
    int nTransitions;
}
stdState;



/*
 * DIAGRAM
 *
 * Definition of a state-transition diagram
 */
typedef struct stdDiagram
{
    // array of states
    stdState *states;

    // number of states and profiles contemplated within the STD
    int nStates;
    int nProfiles;
}
stdDiagram;



/*
 * STD METHODS
 *
 */
stdDiagram *stdCreate(int nStates, int nProfiles);
void        stdDestroy(stdDiagram **std);

int stdAddTransition(stdDiagram *std, int iState, int iProfile, int iNextState);
int stdGetStateEvaluation(stdDiagram *std, int iState, double *transitionEval, double *stateEval);



#endif

