#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inrimage/image.h>
#include "file_tools.h"
#include "inr_io.h"

#define FNAMESIZE    1024

/*** NOT IMPLEMENTED ***/


static char Ucmd[] =
"-c1 sequence_red -c2 sequence_near_infrared -median win \
[-odir output_dir] [-oname output_basename]";

static char Udetail[] =
"\nOutputs filtered image sequences for red and near infra-red input images\n\
\nMedian filtering is applied using a moving window\n";




int main(int argc, char *argv[])
{
  int      okChannel1, okChannel2;
  int      okMedian, okDirOut, okBaseNameOut;
  char     fnameChannel1[FNAMESIZE];
  char     fnameChannel2[FNAMESIZE];
  char     fnameBaseOut[FNAMESIZE];
  char     fnameOut[FNAMESIZE];
  char     dirOut[FNAMESIZE];

  unsigned char *data1;
  unsigned char *data2;
  float         *data_out;
  InrType type1;
  InrType type2;
  int typesize1;
  int typesize2;
  int size_x, size_y, size_z;
  int size2_x, size2_y, size2_z;
  int z;
  int i;
  double val1;
  double val2;
  int    win;
  
  InrGeorefInfo georef;
  

  // initializes inrimage
  inr_init (argc, argv, "1.0", Ucmd, Udetail);

  // checks arguments
  okChannel1    = igetopt1 ("-c1", "%s", fnameChannel1);
  okChannel2    = igetopt1 ("-c2", "%s", fnameChannel2);
  okMedian      = igetopt1 ("-median", "%d", &win);
  okDirOut      = igetopt1 ("-odir"  , "%s", dirOut);
  okBaseNameOut = igetopt1 ("-oname" , "%s", fnameBaseOut);
  
  if (!(okChannel1 && okChannel2))
    iusage_ (Ucmd, Udetail);
  if (tstopts ())
    iusage_ (Ucmd, Udetail);

  if (!inrReadHeader(fnameChannel1, &size_x, &size_y, &size_z, &type1))
  {
    fprintf(stderr, "Error: could not read header for image '%s'.\n", fnameChannel1);
    return 0;
  }

  if (!inrReadHeader(fnameChannel2, &size2_x, &size2_y, &size2_z, &type2))
  {
    fprintf(stderr, "Error: could not read header for image '%s'.\n", fnameChannel2);
    return 0;
  }
  
  if (size_x!=size2_x || size_y!=size2_y || size_z!=size2_z)
  {
    fprintf(stderr, "Error: image sizes do not match.\n");
    return 0;
  }

  typesize1 = InrTypeSizes[type1]/8;
  typesize2 = InrTypeSizes[type2]/8;
  data1    = (unsigned char *)malloc(size_x*size_y*typesize1*sizeof(unsigned char));    
  data2    = (unsigned char *)malloc(size_x*size_y*typesize2*sizeof(unsigned char));
  data_out = (float *)        malloc(size_x*size_y*sizeof(float));
  if (!data1 || !data2 || !data_out)
  {
    fprintf(stderr, "Error: could not allocate memory.\n");
    return 0;
  }
  
  if (!inrWriteHeader(fnameOut, size_x, size_y, size_z, InrFloat))
  {
    fprintf(stderr, "Error: could write to output file '%s'.\n", fnameOut);
    return 0;
  }
  
  for (z=0; z<size_z; z++)
  {
    fprintf(stderr, "Reading layer %d of %d...\r", z+1, size_z);
    if (!inrReadFrame(fnameChannel1, z, data1))
    {
      fprintf(stderr, "Error: could not read frame %d for image '%s'.\n", z, fnameChannel1);
      free(data1);
      free(data2);
      free(data_out);
      return 0;
    }
    if (!inrReadFrame(fnameChannel2, z, data2))
    {
      fprintf(stderr, "Error: could not read frame %d for image '%s'.\n", z, fnameChannel2);
      free(data1);
      free(data2);
      free(data_out);
      return 0;
    }
    fprintf(stderr, "Processing layer %d of %d...\r", z+1, size_z);
    
    for (i=0; i<size_x*size_y; i++)
    {
      val1 = inrGetDataValue(data1, type1, i);
      val2 = inrGetDataValue(data2, type2, i);
      
      if (val1<=0 || val2<=0)
        data_out[i] = 0;
      else
        data_out[i] = (float) ((val2 - val1)/(val2 + val1));
    }
    
    if (!inrWriteFrame(fnameOut, z, (unsigned char *)data_out))
    {
      fprintf(stderr, "Error: could not write frame %d for output image.\n", z);
      free(data1);
      free(data2);
      free(data_out);
      return 0;
    }
  }
  
  if (inrReadGeoref(fnameChannel1, &georef))
    inrWriteGeoref(fnameOut, georef);
  else
    inrWriteEnviHdr(fnameOut);
    
  free(data1);
  free(data2);
  free(data_out);
  printf("NDVI image successfully built.\n");

  return 1;
}

