#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inrimage/image.h>
#include "dirent-universal.h"
#include "file_tools.h"
#include "str_tools.h"
#include "modis.h"
#include "inr_io.h"

#define FLISTSIZE    65536
#define FNAMESIZE    1024


static char Ucmd[] =
"-ifile1 modis_image -ifile2 mask";

static char Udetail[] =
"\nOutputs the mean reflectance value of non-masked pixels of a MODIS image\n";




int main(int argc, char *argv[])
{
  int      okFile1, okFile2;
  char     fnameModis[FNAMESIZE];
  char     fnameMask[FNAMESIZE];

  // initializes inrimage
  inr_init (argc, argv, "1.0", Ucmd, Udetail);

  // checks arguments
  okFile1 = igetopt1 ("-ifile1", "%s", fnameModis);
  okFile2 = igetopt1 ("-ifile2", "%s", fnameMask);
  
  if (!(okFile1 && okFile2))
    iusage_ (Ucmd, Udetail);
  if (tstopts ())
    iusage_ (Ucmd, Udetail);

  {
    unsigned char *data;
    unsigned char *mask;
    InrType typedata;
    int size_x, size_y, size_z;
    int masksize_x, masksize_y;
    int i;
    double value = 0;
    double mean  = 0;
    int    nClear = 0;
    double pClear = 0;
    
    data = inrReadDataSequence(fnameModis, &size_x, &size_y, &size_z, &typedata);
    mask = inrReadByteImage(fnameMask, &masksize_x, &masksize_y);
    
    if (size_x!=masksize_x || size_y!=masksize_y || size_z!=1)
    {
      fprintf(stderr, "Error: image sizes incorrect\n");
      return 1;
    }
    
    for (i=0; i<size_x*size_y; i++)
    {
       value = inrGetDataValue(data, typedata, i);
       if (mask[i] && value>0)
       {
         mean += value;
         nClear++;
       }
    }
    free(data);
    free(mask);
    
    pClear = ((double)nClear) / (size_x*size_y);
    if (pClear < 0.05)
    {
      fprintf(stderr, "Image %s too cloudy.\n", fnameModis);
      return 1;
    }

    mean /= nClear;
    {
      ModisInfo info;
      char filename[FNAMESIZE];
      ftDecodeFileName(fnameModis, NULL, filename);
      
      if (modisParseName(filename, &info))
        printf("%04d ", (info.year - 2000)*366 + info.day);
        
      printf("%lf\n", mean);
    }
  }
  return 0;
}

