#include <stdio.h>
#include <string.h>
#include "std.h"
#include "modis_std.h"



/*
 * MODIS_STD_CREATE
 *
 * Defines the state-transition diagram to be used for MODIS monitoring
 */
stdDiagram *modisStdCreate()
{
    // creates STD structure
    stdDiagram *std = stdCreate(MODIS_STD_NUM_STATES, MODIS_STD_NUM_PROFILES);

    // adds transitions for StateUndefined
    stdAddTransition(std, StateUndefined, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateUndefined, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateUndefined, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateUndefined, ProfPasture,       StatePasture);
    stdAddTransition(std, StateUndefined, ProfSavannah,      StateSavannah);
    stdAddTransition(std, StateUndefined, ProfDeforestation, StateDeforestation);

    // adds transitions for StateAgriculture1
    stdAddTransition(std, StateAgriculture1, ProfAgriculture1, StateAgriculture1);
    stdAddTransition(std, StateAgriculture1, ProfAgriculture2, StateAgriculture2);
    stdAddTransition(std, StateAgriculture1, ProfAgriculture3, StateAgriculture3);
    stdAddTransition(std, StateAgriculture1, ProfPasture,      StatePasture);
    stdAddTransition(std, StateAgriculture1, ProfSavannah,     StateDirtyPasture);

    // adds transitions for StateAgriculture2
    stdAddTransition(std, StateAgriculture2, ProfAgriculture1, StateAgriculture1);
    stdAddTransition(std, StateAgriculture2, ProfAgriculture2, StateAgriculture2);
    stdAddTransition(std, StateAgriculture2, ProfAgriculture3, StateAgriculture3);
    stdAddTransition(std, StateAgriculture2, ProfPasture,      StatePasture);
    stdAddTransition(std, StateAgriculture2, ProfSavannah,     StateDirtyPasture);

    // adds transitions for StateAgriculture3
    stdAddTransition(std, StateAgriculture3, ProfAgriculture1, StateAgriculture1);
    stdAddTransition(std, StateAgriculture3, ProfAgriculture2, StateAgriculture2);
    stdAddTransition(std, StateAgriculture3, ProfAgriculture3, StateAgriculture3);
    stdAddTransition(std, StateAgriculture3, ProfPasture,      StatePasture);
    stdAddTransition(std, StateAgriculture3, ProfSavannah,     StateDirtyPasture);

    // adds transitions for StateSavannah
    stdAddTransition(std, StateSavannah, ProfSavannah,      StateSavannah);
    stdAddTransition(std, StateSavannah, ProfDeforestation, StateDeforestation);

    // adds transitions for StatePasture
    stdAddTransition(std, StatePasture, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StatePasture, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StatePasture, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StatePasture, ProfPasture,       StatePasture);
    stdAddTransition(std, StatePasture, ProfDeforestation, StatePasture);
    stdAddTransition(std, StatePasture, ProfSavannah,      StateDirtyPasture);

    // adds transitions for StateDeforestation
    stdAddTransition(std, StateDeforestation, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateDeforestation, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateDeforestation, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateDeforestation, ProfPasture,       StatePasture);

    // adds transitions for StateDirtyPasture
    stdAddTransition(std, StateDirtyPasture, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateDirtyPasture, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateDirtyPasture, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateDirtyPasture, ProfPasture,       StatePasture);
    stdAddTransition(std, StateDirtyPasture, ProfDeforestation, StatePasture);
    stdAddTransition(std, StateDirtyPasture, ProfSavannah,      StateRegenSavannah1);

    // adds transitions for StateRegenSavannah1
    stdAddTransition(std, StateRegenSavannah1, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateRegenSavannah1, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateRegenSavannah1, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateRegenSavannah1, ProfPasture,       StatePasture);
    stdAddTransition(std, StateRegenSavannah1, ProfDeforestation, StatePasture);
    stdAddTransition(std, StateRegenSavannah1, ProfSavannah,      StateRegenSavannah2);

    // adds transitions for StateRegenSavannah2
    stdAddTransition(std, StateRegenSavannah2, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateRegenSavannah2, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateRegenSavannah2, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateRegenSavannah2, ProfPasture,       StatePasture);
    stdAddTransition(std, StateRegenSavannah2, ProfDeforestation, StatePasture);
    stdAddTransition(std, StateRegenSavannah2, ProfSavannah,      StateRegenSavannah3);

    // adds transitions for StateRegenSavannah3
    stdAddTransition(std, StateRegenSavannah3, ProfAgriculture1,  StateAgriculture1);
    stdAddTransition(std, StateRegenSavannah3, ProfAgriculture2,  StateAgriculture2);
    stdAddTransition(std, StateRegenSavannah3, ProfAgriculture3,  StateAgriculture3);
    stdAddTransition(std, StateRegenSavannah3, ProfPasture,       StatePasture);
    stdAddTransition(std, StateRegenSavannah3, ProfDeforestation, StatePasture);
    stdAddTransition(std, StateRegenSavannah3, ProfSavannah,      StateSavannah);

    // adds transitions for StateUrban
    stdAddTransition(std, StateUrban, ProfPasture, StateUrban);

    return std;
}

