
/*
  donne les dates d'une sequence a partir d'un fichier de liste de dates
  au format yyddd

  JP Berroir, Oct 2002
*/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
//*** MILTON *** 
//#include <locale.h>
#include "noaa_dates.h"

/* fichier ou se trouvent les dates */
//*** MILTON *** filename given in init_date()
#define FNAMESIZE 1024
char listedatesfile[FNAMESIZE];
/*
static char listedatesfile[] =
  "/nfs/data1/Hydrologie/data/ECOAIR/Generated/NOAAsequences/LISTEDATES-0102";
*/
static struct tm *ttm;
static time_t tt;
/* nombre de dates dispo */
static int NDATES;

static char **LISTEDATES;



/* initialise les structures de dates/temps et lit le fichier de dates */
//*** MILTON ***
//void init_date (char *datefile)
int init_date (char *datefile)
{
  FILE *f;
  int i = 0;
  char outdate[10];

  //*** MILTON ***
  //(void) setlocale (LC_ALL, "");
  time (&tt);
  ttm = localtime (&tt);

  //*** MILTON ***
  if (!datefile)
    return 0;
  strcpy(listedatesfile, datefile);

  f = fopen (listedatesfile, "r");
  if (!f)
    {
      fprintf (stderr, "impossible d'ouvrir le fichier de dates %s\n",
	       listedatesfile);
      //*** MILTON ***
	    //exit (1);
	    return 0;
    }

  while (fscanf (f, "%s", outdate) != EOF)
    i++;
  rewind (f);
  NDATES = i;

  LISTEDATES = (char **) malloc (NDATES * sizeof (char *));
  for (i = 0; i < NDATES; i++)
    LISTEDATES[i] = (char *) malloc (10);

  i = 0;
  while (fscanf (f, "%s", outdate) != EOF)
    {
      strcpy (LISTEDATES[i++], outdate);

    }

  fclose (f);
  //*** MILTON ***
  return 1;
}


void
print_frame_date (int frame)
{
  printf ("Frame %d Julian date %s date %s\n",
	  frame, frame_date (frame), julien2date (frame_date (frame)));
}

int
day_difference (int frame1, int frame2)
{
  int date1, date2, aux;
  int yo, year_overlap;
  int dy1, dy2, y2, y1;

  if (frame1 > frame2)
    {
      aux = frame1;
      frame1 = frame2;
      frame2 = aux;
    }
  if (!strptime (LISTEDATES[frame1], "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }
  y1 = ttm->tm_year + 1900;
  dy1 = ttm->tm_yday;
  if (!strptime (LISTEDATES[frame2], "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }
  y2 = ttm->tm_year + 1900;
  dy2 = ttm->tm_yday;

  date1 = atoi (frame_date (frame1));
  date2 = atoi (frame_date (frame2));
  if (date2 - date1 < 0)
    {
      fprintf (stderr, "Error! dates go back in time\n");
      print_frame_date (frame1);
      print_frame_date (frame2);
      exit (1);
    }
  year_overlap = y2 - y1;
  if (!(y2 - y1))
    return dy2 - dy1;

  for (yo = y1; yo < y2; yo++)
    {
      if (isleap (yo))
	dy2 += 366;
      else
	dy2 += 365;
    }
  return dy2 - dy1;

}

int
day_before (int frame)
{
  if (frame <= 0)
    return 0;
  return day_difference (frame, frame - 1);
}

int
day_after (int frame)
{
  if (frame >= NDATES - 1)
    return 0;
  return day_difference (frame, frame + 1);
}


int
day_since_beginning (int frame)
{

  return day_difference (0, frame);
}

int
get_julian_day (int frame)
{
  if (!strptime (LISTEDATES[frame], "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }
  return ttm->tm_yday;

}

int
get_month (int frame)
{
  if (!strptime (LISTEDATES[frame], "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }
  return ttm->tm_mon;
}

int
get_year (int frame)
{
  if (!strptime (LISTEDATES[frame], "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }
  return ttm->tm_year + 1900;

}

void
clean_date ()
{
  int i;
  for (i = 0; i < NDATES; i++)
    free (LISTEDATES[i]);
  free (LISTEDATES);
}

/* conversion julien -> date */
static char *
julien2date (char *in)
{
  char *out;
  out = (char *) malloc (100);
  if (!strptime (in, "%y%j", ttm))
    {
      fprintf (stderr, "Date entree invalide\n");
      exit (1);
    }

  if (strftime (out, (size_t) 100, "%Y %B, %A %d", ttm) == (size_t) 0)
    {
      fprintf (stderr, "Pb dans la conversion de date\n");
      exit (1);
    }
  return out;
}


static char *
frame_date (int frame)
{
  return LISTEDATES[frame];
}
