#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <inrimage/image.h>
#include "inr_io.h"
#include "inr_color.h"

#define FNAMESIZE  1024
#define MIN(x,y) ((x<y)?x:y)
#define MAX(x,y) ((x>y)?x:y)



static char Ucmd[] = "-ifile image -classlegend legend_classification_file -colorlegend color_legend_file -nclass number_of_class -ofile image_output \n";

static char Udetail[] =
"Color a given input INRIMAGE image file, given a text legend class and a text color.\n\
\nThe classification legend file should contain the name of each class.\n\
\nthe color legend file should countain the name of all the class and the corresponding color in RGB format.\n\
\nInput:\n\
\t-ifile\tFile name of the input classification image\n\
\t-classlegend\tFile name of the text file defining the name of the class\n\
\t-nclass\tNumber of class\n\
\t-colorlegend\tFile name of the text file defining the name and the corresponding color of each class\n\
\nOutput:\n\
\t-ofile\tFile name of the ouput colored  classification image\n";

int main (int argc, char **argv)
{

  char fnameInput[FNAMESIZE];
  char fnameOutput[FNAMESIZE];
  char fnameClass[FNAMESIZE];
  char fnameColor[FNAMESIZE];
  int okLegendclass;
  double nClass;
  InrType type;
  InrGeorefInfo georef;
  int size_x, size_y, size_z;
  unsigned char* data;

  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-colorlegend"  , "%s", fnameColor))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-nclass", "%lf", &nClass))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile", "%s", fnameOutput))
    iusage_ (Ucmd, Udetail);
  okLegendclass  = igetopt1 ("-classlegend", "%s", fnameClass);
  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);

  // reads header information for all input data and checks if they are appropriate
  if (!inrReadHeader(fnameInput, &size_x, &size_y, &size_z, &type))
  {
    fprintf(stderr, "Error: could not read header information for input file '%s'.\n", fnameInput);
    return 0;
  }
  
  // writes output file header (output type is always float)
  if (!inrWriteHeader(fnameOutput, size_x, size_y,size_z ,type))
  {
    fprintf(stderr, "Error: could not write header information for output file '%s'.\n", fnameOutput);
    return 0;
  }
  
  // copy input data and info in output
  data=inrReadDataSequence(fnameInput, &size_x, &size_y, &size_z, &type);
  inrWriteDataSequence(fnameOutput, size_x, size_y, size_z,data, type);
  inrWriteEnviHdr(fnameOutput);
  inrReadGeoref(fnameInput, &georef);
  inrWriteGeoref(fnameOutput, georef);
  

  if(!okLegendclass){
    WriteClassLegend(nClass,fnameClass);}
   

  
  if(! inrWriteColor(fnameOutput,nClass,fnameClass,fnameColor)){
       fprintf(stderr, "Image not colored.\n");
       return 0;
  }

  fprintf(stderr, "Image successfully colored.\n");
  return 1;
}
