#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <inrimage/image.h>
#include "inr_io.h"
#include "linearalgebra.h"


#define FNAMESIZE  1024
#define MIN(x,y) ((x<y)?x:y)
#define MAX(x,y) ((x>y)?x:y)



static char Ucmd[] = "-ifile profiles -ofile coeficients\n";

static char Udetail[] =
"Fits temporal reflectance profiles to a Nth degree polynomial function\n\
\nInput:\n\
\t-ifile\tFile name of the input profile (INRIMAGE file)\n\
\nOutput:\n\
\t-ofile\tFile name of the output file containing the function coeficients\n";


int main (int argc, char **argv)
{
  char fnameInput[FNAMESIZE];
  char fnameOutput[FNAMESIZE];
  InrType type;
  
  unsigned char *data;
  int nClass, nDates, size_z;
  int class, date;
  
  int okDegree;
  int degree;
  int size_matrix;
  int i, j;
  matrice *A;
  matrice *b;
  matrice *A_inv;
  matrice *coef;

  FILE *fpOut;


  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-ofile", "%s", fnameOutput))
    iusage_ (Ucmd, Udetail);
  okDegree = igetopt1 ("-degree", "%d", &degree);
  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);

  if (!okDegree)
    degree = 4;

  // reads input data and checks if it is appropriate
  data = inrReadDataSequence(fnameInput, &nClass, &nDates, &size_z, &type);
  if (!data)
  {
    fprintf(stderr, "Error: could not read input file '%s'.\n", fnameInput);
    return 0;
  }
  if (nDates<2 || size_z!=1)
  {
    fprintf (stderr, "Error: input data should be a single image with each column representing one class profile.\n");
    free(data);
    return 0;
  }
  // opens output data file
  fpOut = fopen(fnameOutput, "wt");
  if (!fpOut)
  {
    fprintf(stderr, "Error: could not open output file '%s'.\n", fnameOutput);
    free(data);
    return 0;
  }

  
  size_matrix = degree + 1;
  
  // processes the data
  for (class=0; class<nClass; class++)
  {
    A = Matrice_Nulle(size_matrix, size_matrix);
    b = Matrice_Nulle(size_matrix, 1);
    
    // fills up matrix A
    for (i=0; i<size_matrix; i++)
    {
      for (j=0; j<size_matrix; j++)
      {
        for (date=0; date<nDates; date++)
        {
          int posA = j + i*size_matrix;
          A->val[posA] += pow(date,i+j);
        }
      }
    }
    
    // fills up vector b
    for (i=0; i<size_matrix; i++)
    {
      for (date=0; date<nDates; date++)
      {
        int posData = class + date*nClass;
        b->val[i] += pow(date,i) * inrGetDataValue(data, type, posData);
      }
    }
    
    // solves system and obtains function coefficients
    A_inv = Inverser_Matrice(A);
    coef  = Multiplier_Matrice(A_inv, b);
    
    // writes output data
    fprintf(fpOut, "Class %02d: ", class+1);
    for (i=0; i<size_matrix; i++)
      fprintf(fpOut, "%10g\t", coef->val[i]);
    fprintf(fpOut, "\n");
    
    sprintf(fnameOutput, "teste.class%02d.txt", class+1);
    FILE *fp;
    fp = fopen(fnameOutput, "wt");
    if (!fp)
      continue;
    for (date=0; date<nDates; date++)
    {
      double value = 0;
      for (i=0; i<size_matrix; i++)
        value += coef->val[i]*pow(date,i);
      fprintf(fp, "%lf\n", value);
    }
    fclose(fp);
    
    free_matrice(&A);
    free_matrice(&b);
    free_matrice(&A_inv);
    free_matrice(&coef);
  }
  
  
  
  fprintf(stderr, "Profile image successfully fitted for %d classes\n", nClass);

  fclose(fpOut);
  free(data);
  return 0;
}
