/***************************************************************************
                          inr_membership_maha.c  -  description
                             -------------------
    Created              : 6 August, 2007
    By                   : (C) 2007 by Carlos de la Torre
    Email                : Carlos.de_la_Torre@inria.fr
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inrimage/image.h>
#include "infra/membership.h"
#include "infra/fuzzyclassdef.h"
#include "infra/inr_io.h"

#define FNAMESIZE         1024
// // // #define STRATSIZE         32
// // // 
// // // // If this constants change, be sure to update the Udetail
// // // #define MAHA      "maha"
// // // #define FUZZY_WA  "fuzzyWA"
// // // #define FUZZY_DR  "fuzzyDR"

static char Ucmd[] = "-ifile inr_input -roi roi_mask -def classes_definitions \
-omemb inr_membership\n";

static char Udetail[] =
"Computes a membership value for every pixel, inside the mask, to every class.\n\
Classes are defined in the 'strategy_conf' configuration file.\n\
The computation of the memberships and the format of the configuration file\n\
depend on the strategy selected.\n\
Input and output files are in INRIMAGE format.\n\
\nInput:\n\
\t-ifile\tFile name of the input data\n\
\t-roi\tFile name of a mask image that defines the region of interest\n\
\t\tMask should be zero for pixels to be masked out and non-zero\n\
\t\totherwise\n\
\t-def\tFilename of the textfile with the strategy and classes definitions.\n\
\nOutput:\n\
\t-omemb\tFile name of the output memberships image\n";

int main (int argc, char **argv)
{
  char fnameInput[FNAMESIZE];
  char fnameDef[FNAMESIZE];
  char fnameROIMask[FNAMESIZE];
  char fnameOutputMemb[FNAMESIZE];
  int res=0;
  int size_x, size_y, size_z;
  InrType typeInput;
  InrType type;
  classdef* def;
  unsigned char *ROIMask;
  
  int i, j,k;

  /* lecture options */
  inr_init (argc, argv, "1.0", Ucmd, Udetail);
  if (!igetopt1 ("-ifile", "%s", fnameInput))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-roi"  , "%s", fnameROIMask))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-def", "%s", fnameDef))
    iusage_ (Ucmd, Udetail);
  if (!igetopt1 ("-omemb", "%s", fnameOutputMemb))
    iusage_ (Ucmd, Udetail);

  if (tstopts ())                /* s'il reste des options : exit avec message */
    iusage_ (Ucmd, Udetail);

  // reads header information for all input data and checks if they are appropriate
  if (!inrReadHeader(fnameInput, &size_x, &size_y, &size_z, &typeInput))
  {
    fprintf(stderr, "Error: could not read header information for input file '%s'.\n", fnameInput);
    return 0;
  }
  if (!inrReadHeader(fnameROIMask, &i, &j, &k, &type))
  {
    fprintf(stderr, "Error: could not read header information for ROI mask '%s'.\n", fnameROIMask);
    return 0;
  }
  if (type != InrByte || k != 1)
  {
    fprintf(stderr, "Error: ROI mask '%s' has to be a single image of type 'Byte'.\n", fnameROIMask);
    return 0;
  }
  if (i != size_x || j != size_y)
  {
    fprintf (stderr, "Error: dimensions of input data and ROI mask do not match.\n");
    return 0;
  }
  
  // reads the class definitions
  def = defCreateFromFile(fnameDef);
  if (!def)
  {
    fprintf (stderr, "Error: Couldn't define classes from %s.\n",fnameDef);
    return 0;
  }

  if (def->nFeatures != size_z)
  {
    fprintf (stderr, "Error: number of features in class definition file (%d) does not match number of layers in the input data (%d).\n",
             def->nFeatures, size_z);
    return 0;
  }
    
  // reads the ROI mask image
  ROIMask = inrReadByteImage (fnameROIMask, &i, &j);
  if (!ROIMask)
  {
    fprintf (stderr, "Error: could not read ROI mask data.\n");
    return 0;
  }

  // writes output file header (output type is always float)
  if (!inrWriteHeader(fnameOutputMemb, size_x, size_y, def->nClasses, InrFloat))
  {
    fprintf(stderr, "Error: could not write header information for memberships file '%s'.\n", fnameOutputMemb);
    return 0;
  }

  // Call the appropiate method based on the chosen strategy
    res = mahalachi(fnameInput, ROIMask, def, fnameOutputMemb, typeInput, size_x, size_y, size_z);
 


  //
  free(ROIMask);
  def = defDestroy(def);
  return 0;
}
